@@ -12,7 +12,7 @@ import sys
 
 def main():
     k, a, b, a1, b1 = 2, 4, 1, 12, 4
-    while True:
+    while 1:
         # Next approximation
         p, q, k = k*k, 2*k+1, k+1
         a, b, a1, b1 = a1, b1, p*a+q*a1, p*b+q*b1
@@ -25,6 +25,7 @@ def main():
 
 def output(d):
     # Use write() to avoid spaces between the digits
+    # Use str() to avoid the 'L'
     sys.stdout.write(str(d))
     # Flush so the output is seen immediately
     sys.stdout.flush()
