@@ -41,11 +41,9 @@ Copyright 2009 Gabriel A. Genellina
 
 """
 
-import os
-import re
+import os, sys
 import csv
-import sys
-
+import re
 from docutils import nodes
 from sphinx.builders import Builder
 
@@ -56,31 +54,20 @@ detect_all = re.compile(ur'''
     (?<!\.)\.\.[ \t]*\w+:  # .. foo: (but NOT ... else:)
     ''', re.UNICODE | re.VERBOSE).finditer
 
-
 class Rule:
     def __init__(self, docname, lineno, issue, line):
-        """A rule for ignoring issues"""
+        "A rule for ignoring issues"
         self.docname = docname # document to which this rule applies
         self.lineno = lineno   # line number in the original source;
                                # this rule matches only near that.
                                # None -> don't care
         self.issue = issue     # the markup fragment that triggered this rule
         self.line = line       # text of the container element (single line only)
-        self.used = False
-
-    def __repr__(self):
-        return '{0.docname},,{0.issue},{0.line}'.format(self)
-
-
-
-class dialect(csv.excel):
-    """Our dialect: uses only linefeed as newline."""
-    lineterminator = '\n'
 
 
 class CheckSuspiciousMarkupBuilder(Builder):
     """
-    Checks for possibly invalid markup that may leak into the output.
+    Checks for possibly invalid markup that may leak into the output
     """
     name = 'suspicious'
 
@@ -89,8 +76,7 @@ class CheckSuspiciousMarkupBuilder(Builder):
         self.log_file_name = os.path.join(self.outdir, 'suspicious.csv')
         open(self.log_file_name, 'w').close()
         # load database of previously ignored issues
-        self.load_rules(os.path.join(os.path.dirname(__file__),
-                                     'susp-ignored.csv'))
+        self.load_rules(os.path.join(os.path.dirname(__file__), 'susp-ignored.csv'))
 
     def get_outdated_docs(self):
         return self.env.found_docs
@@ -99,22 +85,19 @@ class CheckSuspiciousMarkupBuilder(Builder):
         return ''
 
     def prepare_writing(self, docnames):
-        pass
+        ### PYTHON PROJECT SPECIFIC ###
+        for name in set(docnames):
+            if name.split('/', 1)[0] == 'documenting':
+                docnames.remove(name)
+        ### PYTHON PROJECT SPECIFIC ###
 
     def write_doc(self, docname, doctree):
-        # set when any issue is encountered in this document
-        self.any_issue = False
+        self.any_issue = False # set when any issue is encountered in this document
         self.docname = docname
         visitor = SuspiciousVisitor(doctree, self)
         doctree.walk(visitor)
 
     def finish(self):
-        unused_rules = [rule for rule in self.rules if not rule.used]
-        if unused_rules:
-            self.warn('Found %s/%s unused rules:' %
-                      (len(unused_rules), len(self.rules)))
-            for rule in unused_rules:
-                self.info(repr(rule))
         return
 
     def check_issue(self, line, lineno, issue):
@@ -122,7 +105,8 @@ class CheckSuspiciousMarkupBuilder(Builder):
             self.report_issue(line, lineno, issue)
 
     def is_ignored(self, line, lineno, issue):
-        """Determine whether this issue should be ignored."""
+        """Determine whether this issue should be ignored.
+        """
         docname = self.docname
         for rule in self.rules:
             if rule.docname != docname: continue
@@ -139,7 +123,6 @@ class CheckSuspiciousMarkupBuilder(Builder):
             if (rule.lineno is not None) and \
                 abs(rule.lineno - lineno) > 5: continue
             # if it came this far, the rule matched
-            rule.used = True
             return True
         return False
 
@@ -156,11 +139,12 @@ class CheckSuspiciousMarkupBuilder(Builder):
 
     def write_log_entry(self, lineno, issue, text):
         f = open(self.log_file_name, 'ab')
-        writer = csv.writer(f, dialect)
+        writer = csv.writer(f)
         writer.writerow([self.docname.encode('utf-8'),
-                         lineno,
-                         issue.encode('utf-8'),
-                         text.strip().encode('utf-8')])
+                lineno,
+                issue.encode('utf-8'),
+                text.strip().encode('utf-8')])
+        del writer
         f.close()
 
     def load_rules(self, filename):
@@ -175,8 +159,7 @@ class CheckSuspiciousMarkupBuilder(Builder):
         except IOError: return
         for i, row in enumerate(csv.reader(f)):
             if len(row) != 4:
-                raise ValueError(
-                    "wrong format in %s, line %d: %s" % (filename, i+1, row))
+                raise ValueError, "wrong format in %s, line %d: %s" % (filename, i+1, row)
             docname, lineno, issue, text = row
             docname = docname.decode('utf-8')
             if lineno: lineno = int(lineno)
@@ -190,7 +173,7 @@ class CheckSuspiciousMarkupBuilder(Builder):
 
 
 def get_lineno(node):
-    """Obtain line number information for a node."""
+    "Obtain line number information for a node"
     lineno = None
     while lineno is None and node:
         node = node.parent
@@ -215,8 +198,7 @@ def extract_line(text, index):
     """
     p = text.rfind('\n', 0, index) + 1
     q = text.find('\n', index)
-    if q < 0:
-        q = len(text)
+    if q<0: q = len(text)
     return text[p:q]
 
 
@@ -235,6 +217,7 @@ class SuspiciousVisitor(nodes.GenericNodeVisitor):
             self.lastlineno = lineno = max(get_lineno(node) or 0, self.lastlineno)
             seen = set() # don't report the same issue more than only once per line
             for match in detect_all(text):
+                #import pdb; pdb.set_trace()
                 issue = match.group()
                 line = extract_line(text, match.start())
                 if (issue, line) not in seen:
