@@ -2,13 +2,10 @@
 Unit tests for refactor.py.
 """
 
-from __future__ import with_statement
-
 import sys
 import os
-import codecs
 import operator
-import StringIO
+import io
 import tempfile
 import shutil
 import unittest
@@ -48,16 +45,12 @@ class TestRefactoringTool(unittest.TestCase):
         return refactor.RefactoringTool(fixers, options, explicit)
 
     def test_print_function_option(self):
-        rt = self.rt({"print_function" : True})
-        self.assertIs(rt.grammar, pygram.python_grammar_no_print_statement)
-        self.assertIs(rt.driver.grammar,
-                      pygram.python_grammar_no_print_statement)
-
-    def test_write_unchanged_files_option(self):
-        rt = self.rt()
-        self.assertFalse(rt.write_unchanged_files)
-        rt = self.rt({"write_unchanged_files" : True})
-        self.assertTrue(rt.write_unchanged_files)
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", DeprecationWarning)
+            refactor.RefactoringTool(_DEFAULT_FIXERS, {"print_function" : True})
+        self.assertEqual(len(w), 1)
+        msg, = w
+        self.assertTrue(msg.category is DeprecationWarning)
 
     def test_fixer_loading_helpers(self):
         contents = ["explicit", "first", "last", "parrot", "preorder"]
@@ -69,50 +62,42 @@ class TestRefactoringTool(unittest.TestCase):
         self.assertEqual(full_names,
                          ["myfixes.fix_" + name for name in contents])
 
-    def test_detect_future_features(self):
-        run = refactor._detect_future_features
-        fs = frozenset
-        empty = fs()
-        self.assertEqual(run(""), empty)
-        self.assertEqual(run("from __future__ import print_function"),
-                         fs(("print_function",)))
-        self.assertEqual(run("from __future__ import generators"),
-                         fs(("generators",)))
-        self.assertEqual(run("from __future__ import generators, feature"),
-                         fs(("generators", "feature")))
-        inp = "from __future__ import generators, print_function"
-        self.assertEqual(run(inp), fs(("generators", "print_function")))
-        inp ="from __future__ import print_function, generators"
-        self.assertEqual(run(inp), fs(("print_function", "generators")))
-        inp = "from __future__ import (print_function,)"
-        self.assertEqual(run(inp), fs(("print_function",)))
-        inp = "from __future__ import (generators, print_function)"
-        self.assertEqual(run(inp), fs(("generators", "print_function")))
-        inp = "from __future__ import (generators, nested_scopes)"
-        self.assertEqual(run(inp), fs(("generators", "nested_scopes")))
-        inp = """from __future__ import generators
+    def test_detect_future_print(self):
+        run = refactor._detect_future_print
+        self.assertFalse(run(""))
+        self.assertTrue(run("from __future__ import print_function"))
+        self.assertFalse(run("from __future__ import generators"))
+        self.assertFalse(run("from __future__ import generators, feature"))
+        input = "from __future__ import generators, print_function"
+        self.assertTrue(run(input))
+        input ="from __future__ import print_function, generators"
+        self.assertTrue(run(input))
+        input = "from __future__ import (print_function,)"
+        self.assertTrue(run(input))
+        input = "from __future__ import (generators, print_function)"
+        self.assertTrue(run(input))
+        input = "from __future__ import (generators, nested_scopes)"
+        self.assertFalse(run(input))
+        input = """from __future__ import generators
 from __future__ import print_function"""
-        self.assertEqual(run(inp), fs(("generators", "print_function")))
-        invalid = ("from",
-                   "from 4",
-                   "from x",
-                   "from x 5",
-                   "from x im",
-                   "from x import",
-                   "from x import 4",
-                   )
-        for inp in invalid:
-            self.assertEqual(run(inp), empty)
-        inp = "'docstring'\nfrom __future__ import print_function"
-        self.assertEqual(run(inp), fs(("print_function",)))
-        inp = "'docstring'\n'somng'\nfrom __future__ import print_function"
-        self.assertEqual(run(inp), empty)
-        inp = "# comment\nfrom __future__ import print_function"
-        self.assertEqual(run(inp), fs(("print_function",)))
-        inp = "# comment\n'doc'\nfrom __future__ import print_function"
-        self.assertEqual(run(inp), fs(("print_function",)))
-        inp = "class x: pass\nfrom __future__ import print_function"
-        self.assertEqual(run(inp), empty)
+        self.assertTrue(run(input))
+        self.assertFalse(run("from"))
+        self.assertFalse(run("from 4"))
+        self.assertFalse(run("from x"))
+        self.assertFalse(run("from x 5"))
+        self.assertFalse(run("from x im"))
+        self.assertFalse(run("from x import"))
+        self.assertFalse(run("from x import 4"))
+        input = "'docstring'\nfrom __future__ import print_function"
+        self.assertTrue(run(input))
+        input = "'docstring'\n'somng'\nfrom __future__ import print_function"
+        self.assertFalse(run(input))
+        input = "# comment\nfrom __future__ import print_function"
+        self.assertTrue(run(input))
+        input = "# comment\n'doc'\nfrom __future__ import print_function"
+        self.assertTrue(run(input))
+        input = "class x: pass\nfrom __future__ import print_function"
+        self.assertFalse(run(input))
 
     def test_get_headnode_dict(self):
         class NoneFix(fixer_base.BaseFix):
@@ -132,7 +117,7 @@ from __future__ import print_function"""
         self.assertEqual(top_fixes, [with_head, no_head])
         name_fixes = d.pop(token.NAME)
         self.assertEqual(name_fixes, [simple, no_head])
-        for fixes in d.itervalues():
+        for fixes in d.values():
             self.assertEqual(fixes, [no_head])
 
     def test_fixer_loading(self):
@@ -172,7 +157,7 @@ from __future__ import print_function"""
         results = []
         rt = MyRT(_DEFAULT_FIXERS)
         save = sys.stdin
-        sys.stdin = StringIO.StringIO("def parrot(): pass\n\n")
+        sys.stdin = io.StringIO("def parrot(): pass\n\n")
         try:
             rt.refactor_stdin()
         finally:
@@ -182,59 +167,27 @@ from __future__ import print_function"""
                     "<stdin>", False]
         self.assertEqual(results, expected)
 
-    def check_file_refactoring(self, test_file, fixers=_2TO3_FIXERS,
-                               options=None, mock_log_debug=None,
-                               actually_write=True):
-        tmpdir = tempfile.mkdtemp(prefix="2to3-test_refactor")
-        self.addCleanup(shutil.rmtree, tmpdir)
-        # make a copy of the tested file that we can write to
-        shutil.copy(test_file, tmpdir)
-        test_file = os.path.join(tmpdir, os.path.basename(test_file))
-        os.chmod(test_file, 0o644)
-
+    def check_file_refactoring(self, test_file, fixers=_2TO3_FIXERS):
         def read_file():
             with open(test_file, "rb") as fp:
                 return fp.read()
-
         old_contents = read_file()
-        rt = self.rt(fixers=fixers, options=options)
-        if mock_log_debug:
-            rt.log_debug = mock_log_debug
+        rt = self.rt(fixers=fixers)
 
         rt.refactor_file(test_file)
         self.assertEqual(old_contents, read_file())
 
-        if not actually_write:
-            return
-        rt.refactor_file(test_file, True)
-        new_contents = read_file()
-        self.assertNotEqual(old_contents, new_contents)
-        return new_contents
+        try:
+            rt.refactor_file(test_file, True)
+            self.assertNotEqual(old_contents, read_file())
+        finally:
+            with open(test_file, "wb") as fp:
+                fp.write(old_contents)
 
     def test_refactor_file(self):
         test_file = os.path.join(FIXER_DIR, "parrot_example.py")
         self.check_file_refactoring(test_file, _DEFAULT_FIXERS)
 
-    def test_refactor_file_write_unchanged_file(self):
-        test_file = os.path.join(FIXER_DIR, "parrot_example.py")
-        debug_messages = []
-        def recording_log_debug(msg, *args):
-            debug_messages.append(msg % args)
-        self.check_file_refactoring(test_file, fixers=(),
-                                    options={"write_unchanged_files": True},
-                                    mock_log_debug=recording_log_debug,
-                                    actually_write=False)
-        # Testing that it logged this message when write=False was passed is
-        # sufficient to see that it did not bail early after "No changes".
-        message_regex = r"Not writing changes to .*%s%s" % (
-                os.sep, os.path.basename(test_file))
-        for message in debug_messages:
-            if "Not writing changes" in message:
-                self.assertRegexpMatches(message, message_regex)
-                break
-        else:
-            self.fail("%r not matched in %r" % (message_regex, debug_messages))
-
     def test_refactor_dir(self):
         def check(structure, expected):
             def mock_refactor_file(self, f, *args):
@@ -259,7 +212,6 @@ from __future__ import print_function"""
                 "hi.py",
                 ".dumb",
                 ".after.py",
-                "notpy.npy",
                 "sappy"]
         expected = ["hi.py"]
         check(tree, expected)
@@ -271,15 +223,6 @@ from __future__ import print_function"""
         fn = os.path.join(TEST_DATA_DIR, "different_encoding.py")
         self.check_file_refactoring(fn)
 
-    def test_false_file_encoding(self):
-        fn = os.path.join(TEST_DATA_DIR, "false_encoding.py")
-        data = self.check_file_refactoring(fn)
-
-    def test_bom(self):
-        fn = os.path.join(TEST_DATA_DIR, "bom.py")
-        data = self.check_file_refactoring(fn)
-        self.assertTrue(data.startswith(codecs.BOM_UTF8))
-
     def test_crlf_newlines(self):
         old_sep = os.linesep
         os.linesep = "\r\n"
@@ -293,19 +236,21 @@ from __future__ import print_function"""
     def test_refactor_docstring(self):
         rt = self.rt()
 
-        doc = """
->>> example()
-42
-"""
-        out = rt.refactor_docstring(doc, "<test>")
-        self.assertEqual(out, doc)
+        def example():
+            """
+            >>> example()
+            42
+            """
+        out = rt.refactor_docstring(example.__doc__, "<test>")
+        self.assertEqual(out, example.__doc__)
 
-        doc = """
->>> def parrot():
-...      return 43
-"""
-        out = rt.refactor_docstring(doc, "<test>")
-        self.assertNotEqual(out, doc)
+        def parrot():
+            """
+            >>> def parrot():
+            ...      return 43
+            """
+        out = rt.refactor_docstring(parrot.__doc__, "<test>")
+        self.assertNotEqual(out, parrot.__doc__)
 
     def test_explicit(self):
         from myfixes.fix_explicit import FixExplicit
