@@ -1,23 +1,27 @@
 # Test hashlib module
 #
-# $Id: test_hashlib.py 79216 2010-03-21 19:16:28Z georg.brandl $
+# $Id: test_hashlib.py 74384 2009-08-13 08:51:18Z georg.brandl $
 #
-#  Copyright (C) 2005-2010   Gregory P. Smith (greg@krypto.org)
+#  Copyright (C) 2005-2009   Gregory P. Smith (greg@krypto.org)
 #  Licensed to PSF under a Contributor Agreement.
 #
 
 import hashlib
+from io import StringIO
+try:
+    import threading
+except ImportError:
+    threading = None
 import unittest
-from test import test_support
-from test.test_support import _4G, precisionbigmemtest
+from test import support
+from test.support import _4G, precisionbigmemtest
 
 def hexstr(s):
-    import string
-    h = string.hexdigits
+    assert isinstance(s, bytes), repr(s)
+    h = "0123456789abcdef"
     r = ''
-    for c in s:
-        i = ord(c)
-        r = r + h[(i >> 4) & 0xF] + h[i & 0xF]
+    for i in s:
+        r += h[(i >> 4) & 0xF] + h[i & 0xF]
     return r
 
 
@@ -32,18 +36,19 @@ class HashLibTestCase(unittest.TestCase):
         except ValueError:
             pass
         else:
-            self.assert_(0 == "hashlib didn't reject bogus hash name")
+            self.assertTrue(0 == "hashlib didn't reject bogus hash name")
 
     def test_hexdigest(self):
         for name in self.supported_hash_names:
             h = hashlib.new(name)
-            self.assert_(hexstr(h.digest()) == h.hexdigest())
+            assert isinstance(h.digest(), bytes), name
+            self.assertEqual(hexstr(h.digest()), h.hexdigest())
 
 
     def test_large_update(self):
-        aas = 'a' * 128
-        bees = 'b' * 127
-        cees = 'c' * 126
+        aas = b'a' * 128
+        bees = b'b' * 127
+        cees = b'c' * 126
 
         for name in self.supported_hash_names:
             m1 = hashlib.new(name)
@@ -58,20 +63,33 @@ class HashLibTestCase(unittest.TestCase):
     def check(self, name, data, digest):
         # test the direct constructors
         computed = getattr(hashlib, name)(data).hexdigest()
-        self.assert_(computed == digest)
+        self.assertEqual(computed, digest)
         # test the general new() interface
         computed = hashlib.new(name, data).hexdigest()
-        self.assert_(computed == digest)
+        self.assertEqual(computed, digest)
 
+    def check_no_unicode(self, algorithm_name):
+        # Unicode objects are not allowed as input.
+        self.assertRaises(TypeError, getattr(hashlib, algorithm_name), 'spam')
+        self.assertRaises(TypeError, hashlib.new, algorithm_name, 'spam')
+
+    def test_no_unicode(self):
+        self.check_no_unicode('md5')
+        self.check_no_unicode('sha1')
+        self.check_no_unicode('sha224')
+        self.check_no_unicode('sha256')
+        self.check_no_unicode('sha384')
+        self.check_no_unicode('sha512')
 
     def test_case_md5_0(self):
-        self.check('md5', '', 'd41d8cd98f00b204e9800998ecf8427e')
+        self.check('md5', b'', 'd41d8cd98f00b204e9800998ecf8427e')
 
     def test_case_md5_1(self):
-        self.check('md5', 'abc', '900150983cd24fb0d6963f7d28e17f72')
+        self.check('md5', b'abc', '900150983cd24fb0d6963f7d28e17f72')
 
     def test_case_md5_2(self):
-        self.check('md5', 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789',
+        self.check('md5',
+                   b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789',
                    'd174ab98d277d9f5a5611c2c9f419d9f')
 
     @precisionbigmemtest(size=_4G + 5, memuse=1)
@@ -95,19 +113,20 @@ class HashLibTestCase(unittest.TestCase):
     # http://www.itl.nist.gov/div897/pubs/fip180-1.htm
 
     def test_case_sha1_0(self):
-        self.check('sha1', "",
+        self.check('sha1', b"",
                    "da39a3ee5e6b4b0d3255bfef95601890afd80709")
 
     def test_case_sha1_1(self):
-        self.check('sha1', "abc",
+        self.check('sha1', b"abc",
                    "a9993e364706816aba3e25717850c26c9cd0d89d")
 
     def test_case_sha1_2(self):
-        self.check('sha1', "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
+        self.check('sha1',
+                   b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
                    "84983e441c3bd26ebaae4aa1f95129e5e54670f1")
 
     def test_case_sha1_3(self):
-        self.check('sha1', "a" * 1000000,
+        self.check('sha1', b"a" * 1000000,
                    "34aa973cd4c4daa4f61eeb2bdbad27316534016f")
 
 
@@ -116,90 +135,141 @@ class HashLibTestCase(unittest.TestCase):
     # http://csrc.nist.gov/publications/fips/fips180-2/fips180-2.pdf
 
     def test_case_sha224_0(self):
-        self.check('sha224', "",
+        self.check('sha224', b"",
           "d14a028c2a3a2bc9476102bb288234c415a2b01f828ea62ac5b3e42f")
 
     def test_case_sha224_1(self):
-        self.check('sha224', "abc",
+        self.check('sha224', b"abc",
           "23097d223405d8228642a477bda255b32aadbce4bda0b3f7e36c9da7")
 
     def test_case_sha224_2(self):
         self.check('sha224',
-          "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
+          b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
           "75388b16512776cc5dba5da1fd890150b0c6455cb4f58b1952522525")
 
     def test_case_sha224_3(self):
-        self.check('sha224', "a" * 1000000,
+        self.check('sha224', b"a" * 1000000,
           "20794655980c91d8bbb4c1ea97618a4bf03f42581948b2ee4ee7ad67")
 
 
     def test_case_sha256_0(self):
-        self.check('sha256', "",
+        self.check('sha256', b"",
           "e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855")
 
     def test_case_sha256_1(self):
-        self.check('sha256', "abc",
+        self.check('sha256', b"abc",
           "ba7816bf8f01cfea414140de5dae2223b00361a396177a9cb410ff61f20015ad")
 
     def test_case_sha256_2(self):
         self.check('sha256',
-          "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
+          b"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq",
           "248d6a61d20638b8e5c026930c3e6039a33ce45964ff2167f6ecedd419db06c1")
 
     def test_case_sha256_3(self):
-        self.check('sha256', "a" * 1000000,
+        self.check('sha256', b"a" * 1000000,
           "cdc76e5c9914fb9281a1c7e284d73e67f1809a48a497200e046d39ccc7112cd0")
 
 
     def test_case_sha384_0(self):
-        self.check('sha384', "",
+        self.check('sha384', b"",
           "38b060a751ac96384cd9327eb1b1e36a21fdb71114be07434c0cc7bf63f6e1da"+
           "274edebfe76f65fbd51ad2f14898b95b")
 
     def test_case_sha384_1(self):
-        self.check('sha384', "abc",
+        self.check('sha384', b"abc",
           "cb00753f45a35e8bb5a03d699ac65007272c32ab0eded1631a8b605a43ff5bed"+
           "8086072ba1e7cc2358baeca134c825a7")
 
     def test_case_sha384_2(self):
         self.check('sha384',
-                   "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"+
-                   "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
+                   b"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"+
+                   b"hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
           "09330c33f71147e83d192fc782cd1b4753111b173b3b05d22fa08086e3b0f712"+
           "fcc7c71a557e2db966c3e9fa91746039")
 
     def test_case_sha384_3(self):
-        self.check('sha384', "a" * 1000000,
+        self.check('sha384', b"a" * 1000000,
           "9d0e1809716474cb086e834e310a4a1ced149e9c00f248527972cec5704c2a5b"+
           "07b8b3dc38ecc4ebae97ddd87f3d8985")
 
 
     def test_case_sha512_0(self):
-        self.check('sha512', "",
+        self.check('sha512', b"",
           "cf83e1357eefb8bdf1542850d66d8007d620e4050b5715dc83f4a921d36ce9ce"+
           "47d0d13c5d85f2b0ff8318d2877eec2f63b931bd47417a81a538327af927da3e")
 
     def test_case_sha512_1(self):
-        self.check('sha512', "abc",
+        self.check('sha512', b"abc",
           "ddaf35a193617abacc417349ae20413112e6fa4e89a97ea20a9eeee64b55d39a"+
           "2192992a274fc1a836ba3c23a3feebbd454d4423643ce80e2a9ac94fa54ca49f")
 
     def test_case_sha512_2(self):
         self.check('sha512',
-                   "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"+
-                   "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
+                   b"abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn"+
+                   b"hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu",
           "8e959b75dae313da8cf4f72814fc143f8f7779c6eb9f7fa17299aeadb6889018"+
           "501d289e4900f7e4331b99dec4b5433ac7d329eeb6dd26545e96e55b874be909")
 
     def test_case_sha512_3(self):
-        self.check('sha512', "a" * 1000000,
+        self.check('sha512', b"a" * 1000000,
           "e718483d0ce769644e2e42c7bc15b4638e1f98b13b2044285632a803afa973eb"+
           "de0ff244877ea60a4cb0432ce577c31beb009c5c2c49aa2e4eadb217ad8cc09b")
 
+    def test_gil(self):
+        # Check things work fine with an input larger than the size required
+        # for multithreaded operation (which is hardwired to 2048).
+        gil_minsize = 2048
 
+        m = hashlib.md5()
+        m.update(b'1')
+        m.update(b'#' * gil_minsize)
+        m.update(b'1')
+        self.assertEquals(m.hexdigest(), 'cb1e1a2cbc80be75e19935d621fb9b21')
+
+        m = hashlib.md5(b'x' * gil_minsize)
+        self.assertEquals(m.hexdigest(), 'cfb767f225d58469c5de3632a8803958')
+
+    def test_threaded_hashing(self):
+        if not threading:
+            raise unittest.SkipTest('No threading module.')
+
+        # Updating the same hash object from several threads at once
+        # using data chunk sizes containing the same byte sequences.
+        #
+        # If the internal locks are working to prevent multiple
+        # updates on the same object from running at once, the resulting
+        # hash will be the same as doing it single threaded upfront.
+        hasher = hashlib.sha1()
+        num_threads = 5
+        smallest_data = b'swineflu'
+        data = smallest_data*200000
+        expected_hash = hashlib.sha1(data*num_threads).hexdigest()
+
+        def hash_in_chunks(chunk_size, event):
+            index = 0
+            while index < len(data):
+                hasher.update(data[index:index+chunk_size])
+                index += chunk_size
+            event.set()
+
+        events = []
+        for threadnum in range(num_threads):
+            chunk_size = len(data) // (10**threadnum)
+            assert chunk_size > 0
+            assert chunk_size % len(smallest_data) == 0
+            event = threading.Event()
+            events.append(event)
+            threading.Thread(target=hash_in_chunks,
+                             args=(chunk_size, event)).start()
+
+        for event in events:
+            event.wait()
+
+        self.assertEqual(expected_hash, hasher.hexdigest())
+
+@support.reap_threads
 def test_main():
-    test_support.run_unittest(HashLibTestCase)
-
+    support.run_unittest(HashLibTestCase)
 
 if __name__ == "__main__":
     test_main()
