@@ -3,15 +3,12 @@ StringIO -- for unicode strings
 BytesIO -- for bytes
 """
 
-from __future__ import unicode_literals
-from __future__ import print_function
-
 import unittest
-from test import test_support as support
+from test import support
 
 import io
 import _pyio as pyio
-import pickle
+import sys
 
 class MemorySeekTestMixin:
 
@@ -23,17 +20,17 @@ class MemorySeekTestMixin:
         buf = self.buftype("1234567890")
         bytesIo = self.ioclass(buf)
 
-        self.assertEqual(buf[:1], bytesIo.read(1))
-        self.assertEqual(buf[1:5], bytesIo.read(4))
-        self.assertEqual(buf[5:], bytesIo.read(900))
-        self.assertEqual(self.EOF, bytesIo.read())
+        self.assertEquals(buf[:1], bytesIo.read(1))
+        self.assertEquals(buf[1:5], bytesIo.read(4))
+        self.assertEquals(buf[5:], bytesIo.read(900))
+        self.assertEquals(self.EOF, bytesIo.read())
 
     def testReadNoArgs(self):
         buf = self.buftype("1234567890")
         bytesIo = self.ioclass(buf)
 
-        self.assertEqual(buf, bytesIo.read())
-        self.assertEqual(self.EOF, bytesIo.read())
+        self.assertEquals(buf, bytesIo.read())
+        self.assertEquals(self.EOF, bytesIo.read())
 
     def testSeek(self):
         buf = self.buftype("1234567890")
@@ -41,21 +38,21 @@ class MemorySeekTestMixin:
 
         bytesIo.read(5)
         bytesIo.seek(0)
-        self.assertEqual(buf, bytesIo.read())
+        self.assertEquals(buf, bytesIo.read())
 
         bytesIo.seek(3)
-        self.assertEqual(buf[3:], bytesIo.read())
+        self.assertEquals(buf[3:], bytesIo.read())
         self.assertRaises(TypeError, bytesIo.seek, 0.0)
 
     def testTell(self):
         buf = self.buftype("1234567890")
         bytesIo = self.ioclass(buf)
 
-        self.assertEqual(0, bytesIo.tell())
+        self.assertEquals(0, bytesIo.tell())
         bytesIo.seek(5)
-        self.assertEqual(5, bytesIo.tell())
+        self.assertEquals(5, bytesIo.tell())
         bytesIo.seek(10000)
-        self.assertEqual(10000, bytesIo.tell())
+        self.assertEquals(10000, bytesIo.tell())
 
 
 class MemoryTestMixin:
@@ -75,7 +72,7 @@ class MemoryTestMixin:
         self.assertEqual(f.seek(0), 0)
         self.assertEqual(f.write(t("h")), 1)
         self.assertEqual(f.truncate(12), 12)
-        self.assertEqual(f.tell(), 1)
+        self.assertEqual(f.tell(), 12)
 
     def test_write(self):
         buf = self.buftype("hello world\n")
@@ -123,11 +120,7 @@ class MemoryTestMixin:
         self.assertEqual(memio.getvalue(), buf[:6])
         self.assertEqual(memio.truncate(4), 4)
         self.assertEqual(memio.getvalue(), buf[:4])
-        # truncate() accepts long objects
-        self.assertEqual(memio.truncate(4L), 4)
-        self.assertEqual(memio.getvalue(), buf[:4])
-        self.assertEqual(memio.tell(), 6)
-        memio.seek(0, 2)
+        self.assertEqual(memio.tell(), 4)
         memio.write(buf)
         self.assertEqual(memio.getvalue(), buf[:4] + buf)
         pos = memio.tell()
@@ -147,6 +140,7 @@ class MemoryTestMixin:
         self.assertEqual(memio.getvalue(), buf * 2)
         memio.__init__(buf)
         self.assertEqual(memio.getvalue(), buf)
+        self.assertRaises(TypeError, memio.__init__, [])
 
     def test_read(self):
         buf = self.buftype("1234567890")
@@ -154,8 +148,7 @@ class MemoryTestMixin:
 
         self.assertEqual(memio.read(0), self.EOF)
         self.assertEqual(memio.read(1), buf[:1])
-        # read() accepts long objects
-        self.assertEqual(memio.read(4L), buf[1:5])
+        self.assertEqual(memio.read(4), buf[1:5])
         self.assertEqual(memio.read(900), buf[5:])
         self.assertEqual(memio.read(), self.EOF)
         memio.seek(0)
@@ -184,8 +177,7 @@ class MemoryTestMixin:
         self.assertEqual(memio.readline(), self.EOF)
         memio.seek(0)
         self.assertEqual(memio.readline(5), buf[:5])
-        # readline() accepts long objects
-        self.assertEqual(memio.readline(5L), buf[5:10])
+        self.assertEqual(memio.readline(5), buf[5:10])
         self.assertEqual(memio.readline(5), buf[10:15])
         memio.seek(0)
         self.assertEqual(memio.readline(-1), buf)
@@ -213,8 +205,7 @@ class MemoryTestMixin:
         memio.seek(5)
         self.assertEqual(memio.readlines(), [buf[5:]] + [buf] * 9)
         memio.seek(0)
-        # readlines() accepts long objects
-        self.assertEqual(memio.readlines(15L), [buf] * 2)
+        self.assertEqual(memio.readlines(15), [buf] * 2)
         memio.seek(0)
         self.assertEqual(memio.readlines(-1), [buf] * 10)
         memio.seek(0)
@@ -233,7 +224,7 @@ class MemoryTestMixin:
 
         self.assertEqual(iter(memio), memio)
         self.assertTrue(hasattr(memio, '__iter__'))
-        self.assertTrue(hasattr(memio, 'next'))
+        self.assertTrue(hasattr(memio, '__next__'))
         i = 0
         for line in memio:
             self.assertEqual(line, buf)
@@ -247,7 +238,7 @@ class MemoryTestMixin:
         self.assertEqual(i, 10)
         memio = self.ioclass(buf * 2)
         memio.close()
-        self.assertRaises(ValueError, next, memio)
+        self.assertRaises(ValueError, memio.__next__)
 
     def test_getvalue(self):
         buf = self.buftype("1234567890")
@@ -275,8 +266,6 @@ class MemoryTestMixin:
         self.assertEqual(memio.seek(0, 0), 0)
         self.assertEqual(memio.read(), buf)
         self.assertEqual(memio.seek(3), 3)
-        # seek() accepts long objects
-        self.assertEqual(memio.seek(3L), 3)
         self.assertEqual(memio.seek(0, 1), 3)
         self.assertEqual(memio.read(), buf[3:])
         self.assertEqual(memio.seek(len(buf)), len(buf))
@@ -328,9 +317,9 @@ class MemoryTestMixin:
         self.assertEqual(memio.isatty(), False)
         self.assertEqual(memio.closed, False)
         memio.close()
-        self.assertRaises(ValueError, memio.writable)
-        self.assertRaises(ValueError, memio.readable)
-        self.assertRaises(ValueError, memio.seekable)
+        self.assertEqual(memio.writable(), True)
+        self.assertEqual(memio.readable(), True)
+        self.assertEqual(memio.seekable(), True)
         self.assertRaises(ValueError, memio.isatty)
         self.assertEqual(memio.closed, True)
 
@@ -357,43 +346,6 @@ class MemoryTestMixin:
             memio = self.ioclass()
             memio.foo = 1
 
-    def test_pickling(self):
-        buf = self.buftype("1234567890")
-        memio = self.ioclass(buf)
-        memio.foo = 42
-        memio.seek(2)
-
-        class PickleTestMemIO(self.ioclass):
-            def __init__(me, initvalue, foo):
-                self.ioclass.__init__(me, initvalue)
-                me.foo = foo
-            # __getnewargs__ is undefined on purpose. This checks that PEP 307
-            # is used to provide pickling support.
-
-        # Pickle expects the class to be on the module level. Here we use a
-        # little hack to allow the PickleTestMemIO class to derive from
-        # self.ioclass without having to define all combinations explicitly on
-        # the module-level.
-        import __main__
-        PickleTestMemIO.__module__ = '__main__'
-        PickleTestMemIO.__qualname__ = PickleTestMemIO.__name__
-        __main__.PickleTestMemIO = PickleTestMemIO
-        submemio = PickleTestMemIO(buf, 80)
-        submemio.seek(2)
-
-        # We only support pickle protocol 2 and onward since we use extended
-        # __reduce__ API of PEP 307 to provide pickling support.
-        for proto in range(2, pickle.HIGHEST_PROTOCOL + 1):
-            for obj in (memio, submemio):
-                obj2 = pickle.loads(pickle.dumps(obj, protocol=proto))
-                self.assertEqual(obj.getvalue(), obj2.getvalue())
-                self.assertEqual(obj.__class__, obj2.__class__)
-                self.assertEqual(obj.foo, obj2.foo)
-                self.assertEqual(obj.tell(), obj2.tell())
-                obj.close()
-                self.assertRaises(ValueError, pickle.dumps, obj, proto)
-        del __main__.PickleTestMemIO
-
 
 class PyBytesIOTest(MemoryTestMixin, MemorySeekTestMixin, unittest.TestCase):
 
@@ -433,17 +385,12 @@ class PyBytesIOTest(MemoryTestMixin, MemorySeekTestMixin, unittest.TestCase):
         self.assertEqual(b, b"")
         self.assertRaises(TypeError, memio.readinto, '')
         import array
-        a = array.array(b'b', b"hello world")
+        a = array.array('b', b"hello world")
         memio = self.ioclass(buf)
         memio.readinto(a)
         self.assertEqual(a.tostring(), b"1234567890d")
         memio.close()
         self.assertRaises(ValueError, memio.readinto, b)
-        memio = self.ioclass(b"123")
-        b = bytearray()
-        memio.seek(42)
-        memio.readinto(b)
-        self.assertEqual(b, b"")
 
     def test_relative_seek(self):
         buf = self.buftype("1234567890")
@@ -471,7 +418,7 @@ class PyBytesIOTest(MemoryTestMixin, MemorySeekTestMixin, unittest.TestCase):
     def test_bytes_array(self):
         buf = b"1234567890"
         import array
-        a = array.array(b'b', buf)
+        a = array.array('b', list(buf))
         memio = self.ioclass(a)
         self.assertEqual(memio.getvalue(), buf)
         self.assertEqual(memio.write(a), 10)
@@ -482,8 +429,13 @@ class PyBytesIOTest(MemoryTestMixin, MemorySeekTestMixin, unittest.TestCase):
         self.ioclass(initial_bytes=buf)
         self.assertRaises(TypeError, self.ioclass, buf, foo=None)
 
+class PyStringIOTest(MemoryTestMixin, MemorySeekTestMixin, unittest.TestCase):
+    buftype = str
+    ioclass = pyio.StringIO
+    UnsupportedOperation = pyio.UnsupportedOperation
+    EOF = ""
 
-class TextIOTestMixin:
+    # TextIO-specific behaviour.
 
     def test_newlines_property(self):
         memio = self.ioclass(newline=None)
@@ -523,17 +475,6 @@ class TextIOTestMixin:
         self.assertIsNone(memio.errors)
         self.assertFalse(memio.line_buffering)
 
-    def test_newline_default(self):
-        memio = self.ioclass("a\nb\r\nc\rd")
-        self.assertEqual(list(memio), ["a\n", "b\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
-
-        memio = self.ioclass()
-        self.assertEqual(memio.write("a\nb\r\nc\rd"), 8)
-        memio.seek(0)
-        self.assertEqual(list(memio), ["a\n", "b\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
-
     def test_newline_none(self):
         # newline=None
         memio = self.ioclass("a\nb\r\nc\rd", newline=None)
@@ -543,16 +484,12 @@ class TextIOTestMixin:
         self.assertEqual(memio.read(2), "\nb")
         self.assertEqual(memio.read(2), "\nc")
         self.assertEqual(memio.read(1), "\n")
-        self.assertEqual(memio.getvalue(), "a\nb\nc\nd")
-
         memio = self.ioclass(newline=None)
         self.assertEqual(2, memio.write("a\n"))
         self.assertEqual(3, memio.write("b\r\n"))
         self.assertEqual(3, memio.write("c\rd"))
         memio.seek(0)
         self.assertEqual(memio.read(), "a\nb\nc\nd")
-        self.assertEqual(memio.getvalue(), "a\nb\nc\nd")
-
         memio = self.ioclass("a\r\nb", newline=None)
         self.assertEqual(memio.read(3), "a\nb")
 
@@ -564,8 +501,6 @@ class TextIOTestMixin:
         self.assertEqual(memio.read(4), "a\nb\r")
         self.assertEqual(memio.read(2), "\nc")
         self.assertEqual(memio.read(1), "\r")
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
-
         memio = self.ioclass(newline="")
         self.assertEqual(2, memio.write("a\n"))
         self.assertEqual(2, memio.write("b\r"))
@@ -573,138 +508,45 @@ class TextIOTestMixin:
         self.assertEqual(2, memio.write("\rd"))
         memio.seek(0)
         self.assertEqual(list(memio), ["a\n", "b\r\n", "c\r", "d"])
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
 
     def test_newline_lf(self):
         # newline="\n"
-        memio = self.ioclass("a\nb\r\nc\rd", newline="\n")
+        memio = self.ioclass("a\nb\r\nc\rd")
         self.assertEqual(list(memio), ["a\n", "b\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
-
-        memio = self.ioclass(newline="\n")
-        self.assertEqual(memio.write("a\nb\r\nc\rd"), 8)
-        memio.seek(0)
-        self.assertEqual(list(memio), ["a\n", "b\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\nb\r\nc\rd")
 
     def test_newline_cr(self):
         # newline="\r"
         memio = self.ioclass("a\nb\r\nc\rd", newline="\r")
+        memio.seek(0)
         self.assertEqual(memio.read(), "a\rb\r\rc\rd")
         memio.seek(0)
         self.assertEqual(list(memio), ["a\r", "b\r", "\r", "c\r", "d"])
-        self.assertEqual(memio.getvalue(), "a\rb\r\rc\rd")
-
-        memio = self.ioclass(newline="\r")
-        self.assertEqual(memio.write("a\nb\r\nc\rd"), 8)
-        memio.seek(0)
-        self.assertEqual(list(memio), ["a\r", "b\r", "\r", "c\r", "d"])
-        memio.seek(0)
-        self.assertEqual(memio.readlines(), ["a\r", "b\r", "\r", "c\r", "d"])
-        self.assertEqual(memio.getvalue(), "a\rb\r\rc\rd")
 
     def test_newline_crlf(self):
         # newline="\r\n"
         memio = self.ioclass("a\nb\r\nc\rd", newline="\r\n")
+        memio.seek(0)
         self.assertEqual(memio.read(), "a\r\nb\r\r\nc\rd")
         memio.seek(0)
         self.assertEqual(list(memio), ["a\r\n", "b\r\r\n", "c\rd"])
-        memio.seek(0)
-        self.assertEqual(memio.readlines(), ["a\r\n", "b\r\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\r\nb\r\r\nc\rd")
-
-        memio = self.ioclass(newline="\r\n")
-        self.assertEqual(memio.write("a\nb\r\nc\rd"), 8)
-        memio.seek(0)
-        self.assertEqual(list(memio), ["a\r\n", "b\r\r\n", "c\rd"])
-        self.assertEqual(memio.getvalue(), "a\r\nb\r\r\nc\rd")
 
     def test_issue5265(self):
         # StringIO can duplicate newlines in universal newlines mode
         memio = self.ioclass("a\r\nb\r\n", newline=None)
         self.assertEqual(memio.read(5), "a\nb\n")
-        self.assertEqual(memio.getvalue(), "a\nb\n")
 
-
-class PyStringIOTest(MemoryTestMixin, MemorySeekTestMixin,
-                     TextIOTestMixin, unittest.TestCase):
-    buftype = unicode
-    ioclass = pyio.StringIO
-    UnsupportedOperation = pyio.UnsupportedOperation
-    EOF = ""
-
-    def test_lone_surrogates(self):
-        # Issue #20424
-        surrogate = unichr(0xd800)
-        memio = self.ioclass(surrogate)
-        self.assertEqual(memio.read(), surrogate)
-
-        memio = self.ioclass()
-        memio.write(surrogate)
-        self.assertEqual(memio.getvalue(), surrogate)
-
-
-class PyStringIOPickleTest(TextIOTestMixin, unittest.TestCase):
-    """Test if pickle restores properly the internal state of StringIO.
-    """
-    buftype = unicode
-    UnsupportedOperation = pyio.UnsupportedOperation
-    EOF = ""
-
-    class ioclass(pyio.StringIO):
-        def __new__(cls, *args, **kwargs):
-            return pickle.loads(pickle.dumps(pyio.StringIO(*args, **kwargs)))
-        def __init__(self, *args, **kwargs):
-            pass
+    def test_newline_argument(self):
+        self.assertRaises(TypeError, self.ioclass, newline=b"\n")
+        self.assertRaises(ValueError, self.ioclass, newline="error")
+        # These should not raise an error
+        for newline in (None, "", "\n", "\r", "\r\n"):
+            self.ioclass(newline=newline)
 
 
 class CBytesIOTest(PyBytesIOTest):
     ioclass = io.BytesIO
     UnsupportedOperation = io.UnsupportedOperation
 
-    test_bytes_array = unittest.skip(
-        "array.array() does not have the new buffer API"
-    )(PyBytesIOTest.test_bytes_array)
-
-
-    def test_getstate(self):
-        memio = self.ioclass()
-        state = memio.__getstate__()
-        self.assertEqual(len(state), 3)
-        bytearray(state[0]) # Check if state[0] supports the buffer interface.
-        self.assertIsInstance(state[1], int)
-        if state[2] is not None:
-            self.assertIsInstance(state[2], dict)
-        memio.close()
-        self.assertRaises(ValueError, memio.__getstate__)
-
-    def test_setstate(self):
-        # This checks whether __setstate__ does proper input validation.
-        memio = self.ioclass()
-        memio.__setstate__((b"no error", 0, None))
-        memio.__setstate__((bytearray(b"no error"), 0, None))
-        memio.__setstate__((b"no error", 0, {'spam': 3}))
-        self.assertRaises(ValueError, memio.__setstate__, (b"", -1, None))
-        self.assertRaises(TypeError, memio.__setstate__, ("unicode", 0, None))
-        self.assertRaises(TypeError, memio.__setstate__, (b"", 0.0, None))
-        self.assertRaises(TypeError, memio.__setstate__, (b"", 0, 0))
-        self.assertRaises(TypeError, memio.__setstate__, (b"len-test", 0))
-        self.assertRaises(TypeError, memio.__setstate__)
-        self.assertRaises(TypeError, memio.__setstate__, 0)
-        memio.close()
-        self.assertRaises(ValueError, memio.__setstate__, (b"closed", 0, None))
-
-    check_sizeof = support.check_sizeof
-
-    @support.cpython_only
-    def test_sizeof(self):
-        basesize = support.calcobjsize(b'P2PP2P')
-        check = self.check_sizeof
-        self.assertEqual(object.__sizeof__(io.BytesIO()), basesize)
-        check(io.BytesIO(), basesize )
-        check(io.BytesIO(b'a'), basesize + 1 + 1 )
-        check(io.BytesIO(b'a' * 1000), basesize + 1000 + 1 )
-
 class CStringIOTest(PyStringIOTest):
     ioclass = io.StringIO
     UnsupportedOperation = io.UnsupportedOperation
@@ -723,51 +565,9 @@ class CStringIOTest(PyStringIOTest):
         self.assertEqual(memio.tell(), len(buf) * 2)
         self.assertEqual(memio.getvalue(), buf + buf)
 
-    def test_getstate(self):
-        memio = self.ioclass()
-        state = memio.__getstate__()
-        self.assertEqual(len(state), 4)
-        self.assertIsInstance(state[0], unicode)
-        self.assertIsInstance(state[1], str)
-        self.assertIsInstance(state[2], int)
-        if state[3] is not None:
-            self.assertIsInstance(state[3], dict)
-        memio.close()
-        self.assertRaises(ValueError, memio.__getstate__)
-
-    def test_setstate(self):
-        # This checks whether __setstate__ does proper input validation.
-        memio = self.ioclass()
-        memio.__setstate__(("no error", "\n", 0, None))
-        memio.__setstate__(("no error", "", 0, {'spam': 3}))
-        self.assertRaises(ValueError, memio.__setstate__, ("", "f", 0, None))
-        self.assertRaises(ValueError, memio.__setstate__, ("", "", -1, None))
-        self.assertRaises(TypeError, memio.__setstate__, (b"", "", 0, None))
-        # trunk is more tolerant than py3k on the type of the newline param
-        #self.assertRaises(TypeError, memio.__setstate__, ("", b"", 0, None))
-        self.assertRaises(TypeError, memio.__setstate__, ("", "", 0.0, None))
-        self.assertRaises(TypeError, memio.__setstate__, ("", "", 0, 0))
-        self.assertRaises(TypeError, memio.__setstate__, ("len-test", 0))
-        self.assertRaises(TypeError, memio.__setstate__)
-        self.assertRaises(TypeError, memio.__setstate__, 0)
-        memio.close()
-        self.assertRaises(ValueError, memio.__setstate__, ("closed", "", 0, None))
-
-
-class CStringIOPickleTest(PyStringIOPickleTest):
-    UnsupportedOperation = io.UnsupportedOperation
-
-    class ioclass(io.StringIO):
-        def __new__(cls, *args, **kwargs):
-            return pickle.loads(pickle.dumps(io.StringIO(*args, **kwargs),
-                                             protocol=2))
-        def __init__(self, *args, **kwargs):
-            pass
-
 
 def test_main():
-    tests = [PyBytesIOTest, PyStringIOTest, CBytesIOTest, CStringIOTest,
-             PyStringIOPickleTest, CStringIOPickleTest]
+    tests = [PyBytesIOTest, PyStringIOTest, CBytesIOTest, CStringIOTest]
     support.run_unittest(*tests)
 
 if __name__ == '__main__':
