@@ -1,14 +1,12 @@
 #!/usr/bin/python
 """
-This script is used to build "official" universal installers on Mac OS X.
-It requires at least Mac OS X 10.4, Xcode 2.2 and the 10.4u SDK for
-32-bit builds.  64-bit or four-way universal builds require at least
-OS X 10.5 and the 10.5 SDK.
+This script is used to build the "official unofficial" universal build on
+Mac OS X. It requires Mac OS X 10.4, Xcode 2.2 and the 10.4u SDK to do its
+work.  64-bit or four-way universal builds require at least OS X 10.5 and
+the 10.5 SDK.
 
-Please ensure that this script keeps working with Python 2.5, to avoid
-bootstrap issues (/usr/bin/python is Python 2.5 on OSX 10.5).  Sphinx,
-which is used to build the documentation, currently requires at least
-Python 2.4.
+Please ensure that this script keeps working with Python 2.3, to avoid
+bootstrap issues (/usr/bin/python is Python 2.3 on OSX 10.4)
 
 Usage: see USAGE variable in the script.
 """
@@ -42,19 +40,16 @@ def grepValue(fn, variable):
         if ln.startswith(variable):
             value = ln[len(variable):].strip()
             return value[1:-1]
-    raise RuntimeError, "Cannot find variable %s" % variable[:-1]
 
 def getVersion():
     return grepValue(os.path.join(SRCDIR, 'configure'), 'PACKAGE_VERSION')
 
-def getVersionTuple():
-    return tuple([int(n) for n in getVersion().split('.')])
-
 def getFullVersion():
     fn = os.path.join(SRCDIR, 'Include', 'patchlevel.h')
     for ln in open(fn):
         if 'PY_VERSION' in ln:
             return ln.split()[-1][1:-1]
+
     raise RuntimeError, "Cannot find full version??"
 
 # The directory we'll use to create the build (will be erased and recreated)
@@ -66,33 +61,12 @@ DEPSRC = os.path.join(WORKDIR, 'third-party')
 DEPSRC = os.path.expanduser('~/Universal/other-sources')
 
 # Location of the preferred SDK
-
-### There are some issues with the SDK selection below here,
-### The resulting binary doesn't work on all platforms that
-### it should. Always default to the 10.4u SDK until that
-### isue is resolved.
-###
-##if int(os.uname()[2].split('.')[0]) == 8:
-##    # Explicitly use the 10.4u (universal) SDK when
-##    # building on 10.4, the system headers are not
-##    # useable for a universal build
-##    SDKPATH = "/Developer/SDKs/MacOSX10.4u.sdk"
-##else:
-##    SDKPATH = "/"
-
 SDKPATH = "/Developer/SDKs/MacOSX10.4u.sdk"
+#SDKPATH = "/"
 
 universal_opts_map = { '32-bit': ('i386', 'ppc',),
                        '64-bit': ('x86_64', 'ppc64',),
-                       'intel':  ('i386', 'x86_64'),
-                       '3-way':  ('ppc', 'i386', 'x86_64'),
-                       'all':    ('i386', 'ppc', 'x86_64', 'ppc64',) }
-default_target_map = {
-        '64-bit': '10.5',
-        '3-way': '10.5',
-        'intel': '10.5',
-        'all': '10.5',
-}
+                       'all': ('i386', 'ppc', 'x86_64', 'ppc64',) }
 
 UNIVERSALOPTS = tuple(universal_opts_map.keys())
 
@@ -100,7 +74,7 @@ UNIVERSALARCHS = '32-bit'
 
 ARCHLIST = universal_opts_map[UNIVERSALARCHS]
 
-# Source directory (assume we're in Mac/BuildScript)
+# Source directory (asume we're in Mac/BuildScript)
 SRCDIR = os.path.dirname(
         os.path.dirname(
             os.path.dirname(
@@ -110,17 +84,6 @@ SRCDIR = os.path.dirname(
 # $MACOSX_DEPLOYMENT_TARGET -> minimum OS X level
 DEPTARGET = '10.3'
 
-target_cc_map = {
-        '10.3': 'gcc-4.0',
-        '10.4': 'gcc-4.0',
-        '10.5': 'gcc-4.0',
-        '10.6': 'gcc-4.2',
-}
-
-CC = target_cc_map[DEPTARGET]
-
-PYTHON_3 = getVersionTuple() >= (3, 0)
-
 USAGE = textwrap.dedent("""\
     Usage: build_python [options]
 
@@ -141,209 +104,177 @@ USAGE = textwrap.dedent("""\
 #   [The recipes are defined here for convenience but instantiated later after
 #    command line options have been processed.]
 def library_recipes():
-    result = []
+    return [
+      dict(
+          name="Bzip2 1.0.4",
+          url="http://www.bzip.org/1.0.4/bzip2-1.0.4.tar.gz",
+          checksum='fc310b254f6ba5fbb5da018f04533688',
+          configure=None,
+          install='make install PREFIX=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
+              shellQuote(os.path.join(WORKDIR, 'libraries')),
+              ' -arch '.join(ARCHLIST),
+              SDKPATH,
+          ),
+      ),
+      dict(
+          name="ZLib 1.2.3",
+          url="http://www.gzip.org/zlib/zlib-1.2.3.tar.gz",
+          checksum='debc62758716a169df9f62e6ab2bc634',
+          configure=None,
+          install='make install prefix=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
+              shellQuote(os.path.join(WORKDIR, 'libraries')),
+              ' -arch '.join(ARCHLIST),
+              SDKPATH,
+          ),
+      ),
+      dict(
+          # Note that GNU readline is GPL'd software
+          name="GNU Readline 5.1.4",
+          url="http://ftp.gnu.org/pub/gnu/readline/readline-5.1.tar.gz" ,
+          checksum='7ee5a692db88b30ca48927a13fd60e46',
+          patchlevel='0',
+          patches=[
+              # The readline maintainers don't do actual micro releases, but
+              # just ship a set of patches.
+              'http://ftp.gnu.org/pub/gnu/readline/readline-5.1-patches/readline51-001',
+              'http://ftp.gnu.org/pub/gnu/readline/readline-5.1-patches/readline51-002',
+              'http://ftp.gnu.org/pub/gnu/readline/readline-5.1-patches/readline51-003',
+              'http://ftp.gnu.org/pub/gnu/readline/readline-5.1-patches/readline51-004',
+          ]
+      ),
 
-    if DEPTARGET < '10.5':
-        result.extend([
-          dict(
-              name="Bzip2 1.0.6",
-              url="http://bzip.org/1.0.6/bzip2-1.0.6.tar.gz",
-              checksum='00b516f4704d4a7cb50a1d97e6e8e15b',
-              configure=None,
-              install='make install CC=%s PREFIX=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
-                  CC,
-                  shellQuote(os.path.join(WORKDIR, 'libraries')),
-                  ' -arch '.join(ARCHLIST),
-                  SDKPATH,
-              ),
-          ),
-          dict(
-              name="ZLib 1.2.3",
-              url="http://www.gzip.org/zlib/zlib-1.2.3.tar.gz",
-              checksum='debc62758716a169df9f62e6ab2bc634',
-              configure=None,
-              install='make install CC=%s prefix=%s/usr/local/ CFLAGS="-arch %s -isysroot %s"'%(
-                  CC,
-                  shellQuote(os.path.join(WORKDIR, 'libraries')),
-                  ' -arch '.join(ARCHLIST),
-                  SDKPATH,
-              ),
-          ),
-          dict(
-              # Note that GNU readline is GPL'd software
-              name="GNU Readline 6.1.2",
-              url="http://ftp.gnu.org/pub/gnu/readline/readline-6.1.tar.gz" ,
-              checksum='fc2f7e714fe792db1ce6ddc4c9fb4ef3',
-              patchlevel='0',
-              patches=[
-                  # The readline maintainers don't do actual micro releases, but
-                  # just ship a set of patches.
-                  'http://ftp.gnu.org/pub/gnu/readline/readline-6.1-patches/readline61-001',
-                  'http://ftp.gnu.org/pub/gnu/readline/readline-6.1-patches/readline61-002',
-              ]
-          ),
-          dict(
-              name="SQLite 3.7.4",
-              url="http://www.sqlite.org/sqlite-autoconf-3070400.tar.gz",
-              checksum='8f0c690bfb33c3cbbc2471c3d9ba0158',
-              configure_env=('CFLAGS="-Os'
-                                  ' -DSQLITE_ENABLE_FTS3'
-                                  ' -DSQLITE_ENABLE_FTS3_PARENTHESIS'
-                                  ' -DSQLITE_ENABLE_RTREE'
-                                  ' -DSQLITE_TCL=0'
-                                  '"'),
-              configure_pre=[
-                  '--enable-threadsafe',
-                  '--enable-shared=no',
-                  '--enable-static=yes',
-                  '--disable-readline',
-                  '--disable-dependency-tracking',
-              ]
-          ),
-          dict(
-              name="NCurses 5.5",
-              url="http://ftp.gnu.org/pub/gnu/ncurses/ncurses-5.5.tar.gz",
-              checksum='e73c1ac10b4bfc46db43b2ddfd6244ef',
-              configure_pre=[
-                  "--enable-widec",
-                  "--without-cxx",
-                  "--without-ada",
-                  "--without-progs",
-                  "--without-curses-h",
-                  "--enable-shared",
-                  "--with-shared",
-                  "--datadir=/usr/share",
-                  "--sysconfdir=/etc",
-                  "--sharedstatedir=/usr/com",
-                  "--with-terminfo-dirs=/usr/share/terminfo",
-                  "--with-default-terminfo-dir=/usr/share/terminfo",
-                  "--libdir=/Library/Frameworks/Python.framework/Versions/%s/lib"%(getVersion(),),
-                  "--enable-termcap",
-              ],
-              patches=[
-                  "ncurses-5.5.patch",
-              ],
-              useLDFlags=False,
-              install='make && make install DESTDIR=%s && cd %s/usr/local/lib && ln -fs ../../../Library/Frameworks/Python.framework/Versions/%s/lib/lib* .'%(
-                  shellQuote(os.path.join(WORKDIR, 'libraries')),
-                  shellQuote(os.path.join(WORKDIR, 'libraries')),
-                  getVersion(),
-                  ),
-          ),
-        ])
-
-    if not PYTHON_3:
-        result.extend([
-          dict(
-              name="Sleepycat DB 4.7.25",
-              url="http://download.oracle.com/berkeley-db/db-4.7.25.tar.gz",
-              checksum='ec2b87e833779681a0c3a814aa71359e',
-              buildDir="build_unix",
-              configure="../dist/configure",
-              configure_pre=[
-                  '--includedir=/usr/local/include/db4',
-              ]
-          ),
-        ])
-
-    return result
+      dict(
+          name="SQLite 3.6.11",
+          url="http://www.sqlite.org/sqlite-3.6.11.tar.gz",
+          checksum='7ebb099696ab76cc6ff65dd496d17858',
+          configure_pre=[
+              '--enable-threadsafe',
+              '--enable-tempstore',
+              '--enable-shared=no',
+              '--enable-static=yes',
+              '--disable-tcl',
+          ]
+      ),
 
+    dict(
+        name="NCurses 5.5",
+        url="http://ftp.gnu.org/pub/gnu/ncurses/ncurses-5.5.tar.gz",
+        checksum='e73c1ac10b4bfc46db43b2ddfd6244ef',
+        configure_pre=[
+            "--without-cxx",
+            "--without-ada",
+            "--without-progs",
+            "--without-curses-h",
+            "--enable-shared",
+            "--with-shared",
+            "--datadir=/usr/share",
+            "--sysconfdir=/etc",
+            "--sharedstatedir=/usr/com",
+            "--with-terminfo-dirs=/usr/share/terminfo",
+            "--with-default-terminfo-dir=/usr/share/terminfo",
+            "--libdir=/Library/Frameworks/Python.framework/Versions/%s/lib"%(getVersion(),),
+            "--enable-termcap",
+        ],
+        patches=[
+            "ncurses-5.5.patch",
+        ],
+        useLDFlags=False,
+        install='make && make install DESTDIR=%s && cd %s/usr/local/lib && ln -fs ../../../Library/Frameworks/Python.framework/Versions/%s/lib/lib* .'%(
+            shellQuote(os.path.join(WORKDIR, 'libraries')),
+            shellQuote(os.path.join(WORKDIR, 'libraries')),
+            getVersion(),
+            ),
+    ),
+]
 
 # Instructions for building packages inside the .mpkg.
-def pkg_recipes():
-    unselected_for_python3 = ('selected', 'unselected')[PYTHON_3]
-    result = [
-        dict(
-            name="PythonFramework",
-            long_name="Python Framework",
-            source="/Library/Frameworks/Python.framework",
-            readme="""\
-                This package installs Python.framework, that is the python
-                interpreter and the standard library. This also includes Python
-                wrappers for lots of Mac OS X API's.
+PKG_RECIPES = [
+    dict(
+        name="PythonFramework",
+        long_name="Python Framework",
+        source="/Library/Frameworks/Python.framework",
+        readme="""\
+            This package installs Python.framework, that is the python
+            interpreter and the standard library. This also includes Python
+            wrappers for lots of Mac OS X API's.
+        """,
+        postflight="scripts/postflight.framework",
+        selected='selected',
+    ),
+    dict(
+        name="PythonApplications",
+        long_name="GUI Applications",
+        source="/Applications/Python %(VER)s",
+        readme="""\
+            This package installs IDLE (an interactive Python IDE),
+            Python Launcher and Build Applet (create application bundles
+            from python scripts).
+
+            It also installs a number of examples and demos.
             """,
-            postflight="scripts/postflight.framework",
-            selected='selected',
-        ),
-        dict(
-            name="PythonApplications",
-            long_name="GUI Applications",
-            source="/Applications/Python %(VER)s",
-            readme="""\
-                This package installs IDLE (an interactive Python IDE),
-                Python Launcher and Build Applet (create application bundles
-                from python scripts).
+        required=False,
+        selected='selected',
+    ),
+    dict(
+        name="PythonUnixTools",
+        long_name="UNIX command-line tools",
+        source="/usr/local/bin",
+        readme="""\
+            This package installs the unix tools in /usr/local/bin for
+            compatibility with older releases of Python. This package
+            is not necessary to use Python.
+            """,
+        required=False,
+        selected='selected',
+    ),
+    dict(
+        name="PythonDocumentation",
+        long_name="Python Documentation",
+        topdir="/Library/Frameworks/Python.framework/Versions/%(VER)s/Resources/English.lproj/Documentation",
+        source="/pydocs",
+        readme="""\
+            This package installs the python documentation at a location
+            that is useable for pydoc and IDLE. If you have installed Xcode
+            it will also install a link to the documentation in
+            /Developer/Documentation/Python
+            """,
+        postflight="scripts/postflight.documentation",
+        required=False,
+        selected='selected',
+    ),
+    dict(
+        name="PythonProfileChanges",
+        long_name="Shell profile updater",
+        readme="""\
+            This packages updates your shell profile to make sure that
+            the Python tools are found by your shell in preference of
+            the system provided Python tools.
 
-                It also installs a number of examples and demos.
-                """,
-            required=False,
-            selected='selected',
-        ),
-        dict(
-            name="PythonUnixTools",
-            long_name="UNIX command-line tools",
-            source="/usr/local/bin",
-            readme="""\
-                This package installs the unix tools in /usr/local/bin for
-                compatibility with older releases of Python. This package
-                is not necessary to use Python.
-                """,
-            required=False,
-            selected='selected',
-        ),
-        dict(
-            name="PythonDocumentation",
-            long_name="Python Documentation",
-            topdir="/Library/Frameworks/Python.framework/Versions/%(VER)s/Resources/English.lproj/Documentation",
-            source="/pydocs",
-            readme="""\
-                This package installs the python documentation at a location
-                that is useable for pydoc and IDLE. If you have installed Xcode
-                it will also install a link to the documentation in
-                /Developer/Documentation/Python
-                """,
-            postflight="scripts/postflight.documentation",
-            required=False,
-            selected='selected',
-        ),
-        dict(
-            name="PythonProfileChanges",
-            long_name="Shell profile updater",
-            readme="""\
-                This packages updates your shell profile to make sure that
-                the Python tools are found by your shell in preference of
-                the system provided Python tools.
-
-                If you don't install this package you'll have to add
-                "/Library/Frameworks/Python.framework/Versions/%(VER)s/bin"
-                to your PATH by hand.
-                """,
-            postflight="scripts/postflight.patch-profile",
-            topdir="/Library/Frameworks/Python.framework",
-            source="/empty-dir",
-            required=False,
-            selected=unselected_for_python3,
-        ),
-    ]
-
-    if DEPTARGET < '10.4':
-        result.append(
-            dict(
-                name="PythonSystemFixes",
-                long_name="Fix system Python",
-                readme="""\
-                    This package updates the system python installation on
-                    Mac OS X 10.3 to ensure that you can build new python extensions
-                    using that copy of python after installing this version.
-                    """,
-                postflight="../Tools/fixapplepython23.py",
-                topdir="/Library/Frameworks/Python.framework",
-                source="/empty-dir",
-                required=False,
-                selected=unselected_for_python3,
-            )
-        )
-    return result
+            If you don't install this package you'll have to add
+            "/Library/Frameworks/Python.framework/Versions/%(VER)s/bin"
+            to your PATH by hand.
+            """,
+        postflight="scripts/postflight.patch-profile",
+        topdir="/Library/Frameworks/Python.framework",
+        source="/empty-dir",
+        required=False,
+        selected='unselected',
+    ),
+    dict(
+        name="PythonSystemFixes",
+        long_name="Fix system Python",
+        readme="""\
+            This package updates the system python installation on
+            Mac OS X 10.3 to ensure that you can build new python extensions
+            using that copy of python after installing this version.
+            """,
+        postflight="../Tools/fixapplepython23.py",
+        topdir="/Library/Frameworks/Python.framework",
+        source="/empty-dir",
+        required=False,
+        selected='unselected',
+    )
+]
 
 def fatal(msg):
     """
@@ -362,7 +293,7 @@ def fileContents(fn):
 
 def runCommand(commandline):
     """
-    Run a command and raise RuntimeError if it fails. Output is suppressed
+    Run a command and raise RuntimeError if it fails. Output is surpressed
     unless the command fails.
     """
     fd = os.popen(commandline, 'r')
@@ -385,77 +316,21 @@ def captureCommand(commandline):
 
     return data
 
-def getTclTkVersion(configfile, versionline):
-    """
-    search Tcl or Tk configuration file for version line
-    """
-    try:
-        f = open(configfile, "r")
-    except:
-        fatal("Framework configuration file not found: %s" % configfile)
-
-    for l in f:
-        if l.startswith(versionline):
-            f.close()
-            return l
-
-    fatal("Version variable %s not found in framework configuration file: %s"
-            % (versionline, configfile))
-
 def checkEnvironment():
     """
     Check that we're running on a supported system.
     """
 
-    if sys.version_info[0:2] < (2, 4):
-        fatal("This script must be run with Python 2.4 or later")
-
     if platform.system() != 'Darwin':
-        fatal("This script should be run on a Mac OS X 10.4 (or later) system")
+        fatal("This script should be run on a Mac OS X 10.4 system")
 
-    if int(platform.release().split('.')[0]) < 8:
-        fatal("This script should be run on a Mac OS X 10.4 (or later) system")
+    if platform.release() <= '8.':
+        fatal("This script should be run on a Mac OS X 10.4 system")
 
     if not os.path.exists(SDKPATH):
         fatal("Please install the latest version of Xcode and the %s SDK"%(
             os.path.basename(SDKPATH[:-4])))
 
-    # Because we only support dynamic load of only one major/minor version of
-    # Tcl/Tk, ensure:
-    # 1. there are no user-installed frameworks of Tcl/Tk with version
-    #       higher than the Apple-supplied system version
-    # 2. there is a user-installed framework in /Library/Frameworks with the
-    #       same version as the system version.  This allows users to choose
-    #       to install a newer patch level.
-
-    for framework in ['Tcl', 'Tk']:
-        #fw = dict(lower=framework.lower(),
-        #            upper=framework.upper(),
-        #            cap=framework.capitalize())
-        #fwpth = "Library/Frameworks/%(cap)s.framework/%(lower)sConfig.sh" % fw
-        fwpth = 'Library/Frameworks/Tcl.framework/Versions/Current'
-        sysfw = os.path.join(SDKPATH, 'System', fwpth)
-        libfw = os.path.join('/', fwpth)
-        usrfw = os.path.join(os.getenv('HOME'), fwpth)
-        #version = "%(upper)s_VERSION" % fw
-        if os.readlink(libfw) != os.readlink(sysfw):
-            fatal("Version of %s must match %s" % (libfw, sysfw) )
-        if os.path.exists(usrfw):
-            fatal("Please rename %s to avoid possible dynamic load issues."
-                    % usrfw)
-
-    # Remove inherited environment variables which might influence build
-    environ_var_prefixes = ['CPATH', 'C_INCLUDE_', 'DYLD_', 'LANG', 'LC_',
-                            'LD_', 'LIBRARY_', 'PATH', 'PYTHON']
-    for ev in list(os.environ):
-        for prefix in environ_var_prefixes:
-            if ev.startswith(prefix) :
-                print "INFO: deleting environment variable %s=%s" % (
-                                                    ev, os.environ[ev])
-                del os.environ[ev]
-
-    os.environ['PATH'] = '/bin:/sbin:/usr/bin:/usr/sbin'
-    print "Setting default PATH: %s"%(os.environ['PATH'])
 
 
 def parseOptions(args=None):
@@ -463,7 +338,7 @@ def parseOptions(args=None):
     Parse arguments and update global settings.
     """
     global WORKDIR, DEPSRC, SDKPATH, SRCDIR, DEPTARGET
-    global UNIVERSALOPTS, UNIVERSALARCHS, ARCHLIST, CC
+    global UNIVERSALOPTS, UNIVERSALARCHS, ARCHLIST
 
     if args is None:
         args = sys.argv[1:]
@@ -480,7 +355,6 @@ def parseOptions(args=None):
         print "Additional arguments"
         sys.exit(1)
 
-    deptarget = None
     for k, v in options:
         if k in ('-h', '-?', '--help'):
             print USAGE
@@ -500,16 +374,11 @@ def parseOptions(args=None):
 
         elif k in ('--dep-target', ):
             DEPTARGET=v
-            deptarget=v
 
         elif k in ('--universal-archs', ):
             if v in UNIVERSALOPTS:
                 UNIVERSALARCHS = v
                 ARCHLIST = universal_opts_map[UNIVERSALARCHS]
-                if deptarget is None:
-                    # Select alternate default deployment
-                    # target
-                    DEPTARGET = default_target_map.get(v, '10.3')
             else:
                 raise NotImplementedError, v
 
@@ -521,8 +390,6 @@ def parseOptions(args=None):
     SDKPATH=os.path.abspath(SDKPATH)
     DEPSRC=os.path.abspath(DEPSRC)
 
-    CC=target_cc_map[DEPTARGET]
-
     print "Settings:"
     print " * Source directory:", SRCDIR
     print " * Build directory: ", WORKDIR
@@ -530,7 +397,6 @@ def parseOptions(args=None):
     print " * Third-party source:", DEPSRC
     print " * Deployment target:", DEPTARGET
     print " * Universal architectures:", ARCHLIST
-    print " * C compiler:", CC
     print ""
 
 
@@ -708,9 +574,6 @@ def buildRecipe(recipe, basedir, archList):
         configure_args.insert(0, configure)
         configure_args = [ shellQuote(a) for a in configure_args ]
 
-        if 'configure_env' in recipe:
-            configure_args.insert(0, recipe['configure_env'])
-
         print "Running configure for %s"%(name,)
         runCommand(' '.join(configure_args) + ' 2>&1')
 
@@ -749,10 +612,10 @@ def buildPythonDocs():
     curDir = os.getcwd()
     os.chdir(buildDir)
     runCommand('make update')
-    runCommand("make html PYTHON='%s'" % os.path.abspath(sys.executable))
+    runCommand('make html')
     os.chdir(curDir)
-    if not os.path.exists(docdir):
-        os.mkdir(docdir)
+    if os.path.exists(docdir):
+        os.rmdir(docdir)
     os.rename(os.path.join(buildDir, 'build', 'html'), docdir)
 
 
@@ -766,9 +629,9 @@ def buildPython():
         shutil.rmtree(buildDir)
     if os.path.exists(rootDir):
         shutil.rmtree(rootDir)
-    os.makedirs(buildDir)
-    os.makedirs(rootDir)
-    os.makedirs(os.path.join(rootDir, 'empty-dir'))
+    os.mkdir(buildDir)
+    os.mkdir(rootDir)
+    os.mkdir(os.path.join(rootDir, 'empty-dir'))
     curdir = os.getcwd()
     os.chdir(buildDir)
 
@@ -787,20 +650,18 @@ def buildPython():
                                         'libraries', 'usr', 'local', 'lib')
     print "Running configure..."
     runCommand("%s -C --enable-framework --enable-universalsdk=%s "
-               "--with-universal-archs=%s "
-               "%s "
+               "--with-universal-archs=%s --with-computed-gotos "
                "LDFLAGS='-g -L%s/libraries/usr/local/lib' "
                "OPT='-g -O3 -I%s/libraries/usr/local/include' 2>&1"%(
         shellQuote(os.path.join(SRCDIR, 'configure')), shellQuote(SDKPATH),
         UNIVERSALARCHS,
-        (' ', '--with-computed-gotos ')[PYTHON_3],
         shellQuote(WORKDIR)[1:-1],
         shellQuote(WORKDIR)[1:-1]))
 
     print "Running make"
     runCommand("make")
 
-    print "Running make install"
+    print "Running make frameworkinstall"
     runCommand("make install DESTDIR=%s"%(
         shellQuote(rootDir)))
 
@@ -824,6 +685,8 @@ def buildPython():
     frmDir = os.path.join(rootDir, 'Library', 'Frameworks', 'Python.framework')
     gid = grp.getgrnam('admin').gr_gid
 
+
+
     for dirpath, dirnames, filenames in os.walk(frmDir):
         for dn in dirnames:
             os.chmod(os.path.join(dirpath, dn), 0775)
@@ -840,33 +703,12 @@ def buildPython():
             os.chmod(p, stat.S_IMODE(st.st_mode) | stat.S_IWGRP)
             os.chown(p, -1, gid)
 
-    if PYTHON_3:
-        LDVERSION=None
-        VERSION=None
-        ABIFLAGS=None
-
-        fp = open(os.path.join(buildDir, 'Makefile'), 'r')
-        for ln in fp:
-            if ln.startswith('VERSION='):
-                VERSION=ln.split()[1]
-            if ln.startswith('ABIFLAGS='):
-                ABIFLAGS=ln.split()[1]
-            if ln.startswith('LDVERSION='):
-                LDVERSION=ln.split()[1]
-        fp.close()
-
-        LDVERSION = LDVERSION.replace('$(VERSION)', VERSION)
-        LDVERSION = LDVERSION.replace('$(ABIFLAGS)', ABIFLAGS)
-        config_suffix = '-' + LDVERSION
-    else:
-        config_suffix = ''      # Python 2.x
-
     # We added some directories to the search path during the configure
     # phase. Remove those because those directories won't be there on
     # the end-users system.
     path =os.path.join(rootDir, 'Library', 'Frameworks', 'Python.framework',
                 'Versions', version, 'lib', 'python%s'%(version,),
-                'config' + config_suffix, 'Makefile')
+                'config', 'Makefile')
     fp = open(path, 'r')
     data = fp.read()
     fp.close()
@@ -891,18 +733,19 @@ def buildPython():
 
     os.chdir(curdir)
 
-    if PYTHON_3:
-        # Remove the 'Current' link, that way we don't accidently mess
-        # with an already installed version of python 2
-        os.unlink(os.path.join(rootDir, 'Library', 'Frameworks',
-                            'Python.framework', 'Versions', 'Current'))
+    # Remove the 'Current' link, that way we don't accidently mess with an already installed
+    # version of python
+    os.unlink(os.path.join(rootDir, 'Library', 'Frameworks', 'Python.framework', 'Versions', 'Current'))
+
+
+
 
 def patchFile(inPath, outPath):
     data = fileContents(inPath)
     data = data.replace('$FULL_VERSION', getFullVersion())
     data = data.replace('$VERSION', getVersion())
     data = data.replace('$MACOSX_DEPLOYMENT_TARGET', ''.join((DEPTARGET, ' or later')))
-    data = data.replace('$ARCHITECTURES', ", ".join(universal_opts_map[UNIVERSALARCHS]))
+    data = data.replace('$ARCHITECTURES', "i386, ppc")
     data = data.replace('$INSTALL_SIZE', installSize())
 
     # This one is not handy as a template variable
@@ -1029,9 +872,9 @@ def makeMpkgPlist(path):
             IFPkgFlagPackageList=[
                 dict(
                     IFPkgFlagPackageLocation='%s-%s.pkg'%(item['name'], getVersion()),
-                    IFPkgFlagPackageSelection=item.get('selected', 'selected'),
+                    IFPkgFlagPackageSelection=item['selected'],
                 )
-                for item in pkg_recipes()
+                for item in PKG_RECIPES
             ],
             IFPkgFormatVersion=0.10000000149011612,
             IFPkgFlagBackgroundScaling="proportional",
@@ -1058,7 +901,7 @@ def buildInstaller():
     pkgroot = os.path.join(outdir, 'Python.mpkg', 'Contents')
     pkgcontents = os.path.join(pkgroot, 'Packages')
     os.makedirs(pkgcontents)
-    for recipe in pkg_recipes():
+    for recipe in PKG_RECIPES:
         packageFromRecipe(pkgcontents, recipe)
 
     rsrcDir = os.path.join(pkgroot, 'Resources')
@@ -1106,9 +949,9 @@ def buildDMG():
         shutil.rmtree(outdir)
 
     imagepath = os.path.join(outdir,
-                    'python-%s-macosx%s'%(getFullVersion(),DEPTARGET))
+                    'python-%s-macosx'%(getFullVersion(),))
     if INCLUDE_TIMESTAMP:
-        imagepath = imagepath + '-%04d-%02d-%02d'%(time.localtime()[:3])
+        imagepath = imagepath + '%04d-%02d-%02d'%(time.localtime()[:3])
     imagepath = imagepath + '.dmg'
 
     os.mkdir(outdir)
@@ -1147,14 +990,13 @@ def setIcon(filePath, icnsPath):
     Set the custom icon for the specified file or directory.
     """
 
-    dirPath = os.path.normpath(os.path.dirname(__file__))
-    toolPath = os.path.join(dirPath, "seticon.app/Contents/MacOS/seticon")
+    toolPath = os.path.join(os.path.dirname(__file__), "seticon.app/Contents/MacOS/seticon")
+    dirPath = os.path.dirname(__file__)
     if not os.path.exists(toolPath) or os.stat(toolPath).st_mtime < os.stat(dirPath + '/seticon.m').st_mtime:
         # NOTE: The tool is created inside an .app bundle, otherwise it won't work due
         # to connections to the window server.
-        appPath = os.path.join(dirPath, "seticon.app/Contents/MacOS")
-        if not os.path.exists(appPath):
-            os.makedirs(appPath)
+        if not os.path.exists('seticon.app/Contents/MacOS'):
+            os.makedirs('seticon.app/Contents/MacOS')
         runCommand("cc -o %s %s/seticon.m -framework Cocoa"%(
             shellQuote(toolPath), shellQuote(dirPath)))
 
@@ -1167,14 +1009,11 @@ def main():
     checkEnvironment()
 
     os.environ['MACOSX_DEPLOYMENT_TARGET'] = DEPTARGET
-    os.environ['CC'] = CC
 
     if os.path.exists(WORKDIR):
         shutil.rmtree(WORKDIR)
     os.mkdir(WORKDIR)
 
-    os.environ['LC_ALL'] = 'C'
-
     # Then build third-party libraries such as sleepycat DB4.
     buildLibraries()
 
@@ -1216,8 +1055,11 @@ def main():
     print >> fp, "# By:", pwd.getpwuid(os.getuid()).pw_gecos
     fp.close()
 
+
+
     # And copy it to a DMG
     buildDMG()
 
+
 if __name__ == "__main__":
     main()
