@@ -69,7 +69,7 @@ bufferediobase_readinto(PyObject *self, PyObject *args)
 static PyObject *
 bufferediobase_unsupported(const char *message)
 {
-    PyErr_SetString(_PyIO_unsupported_operation, message);
+    PyErr_SetString(IO_STATE->unsupported_operation, message);
     return NULL;
 }
 
@@ -224,7 +224,6 @@ typedef struct {
 
 #ifdef WITH_THREAD
     PyThread_type_lock lock;
-    volatile long owner;
 #endif
 
     Py_ssize_t buffer_size;
@@ -260,39 +259,15 @@ typedef struct {
 /* These macros protect the buffered object against concurrent operations. */
 
 #ifdef WITH_THREAD
-
-static int
-_enter_buffered_busy(buffered *self)
-{
-    if (self->owner == PyThread_get_thread_ident()) {
-        PyObject *r = PyObject_Repr((PyObject *) self);
-        if (r != NULL) {
-            PyErr_Format(PyExc_RuntimeError,
-                         "reentrant call inside %s",
-                         PyString_AS_STRING(r));
-            Py_DECREF(r);
-        }
-        return 0;
-    }
-    Py_BEGIN_ALLOW_THREADS
-    PyThread_acquire_lock(self->lock, 1);
-    Py_END_ALLOW_THREADS
-    return 1;
-}
-
 #define ENTER_BUFFERED(self) \
-    ( (PyThread_acquire_lock(self->lock, 0) ? \
-       1 : _enter_buffered_busy(self)) \
-     && (self->owner = PyThread_get_thread_ident(), 1) )
+    Py_BEGIN_ALLOW_THREADS \
+    PyThread_acquire_lock(self->lock, 1); \
+    Py_END_ALLOW_THREADS
 
 #define LEAVE_BUFFERED(self) \
-    do { \
-        self->owner = 0; \
-        PyThread_release_lock(self->lock); \
-    } while(0);
-
+    PyThread_release_lock(self->lock);
 #else
-#define ENTER_BUFFERED(self) 1
+#define ENTER_BUFFERED(self)
 #define LEAVE_BUFFERED(self)
 #endif
 
@@ -448,8 +423,7 @@ buffered_close(buffered *self, PyObject *args)
     int r;
 
     CHECK_INITIALIZED(self)
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
 
     r = buffered_closed(self);
     if (r < 0)
@@ -462,10 +436,13 @@ buffered_close(buffered *self, PyObject *args)
     /* flush() will most probably re-take the lock, so drop it first */
     LEAVE_BUFFERED(self)
     res = PyObject_CallMethodObjArgs((PyObject *)self, _PyIO_str_flush, NULL);
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
     if (res == NULL) {
-        goto end;
+        /* If flush() fails, just give up */
+        if (PyErr_ExceptionMatches(PyExc_IOError))
+            PyErr_Clear();
+        else
+            goto end;
     }
     Py_XDECREF(res);
 
@@ -550,7 +527,7 @@ buffered_isatty(buffered *self, PyObject *args)
 
 /* Forward decls */
 static PyObject *
-_bufferedwriter_flush_unlocked(buffered *);
+_bufferedwriter_flush_unlocked(buffered *, int);
 static Py_ssize_t
 _bufferedreader_fill_buffer(buffered *self);
 static void
@@ -571,18 +548,6 @@ _bufferedreader_read_generic(buffered *self, Py_ssize_t);
  * Helpers
  */
 
-/* Sets the current error to BlockingIOError */
-static void
-_set_BlockingIOError(char *msg, Py_ssize_t written)
-{
-    PyObject *err;
-    err = PyObject_CallFunction(PyExc_BlockingIOError, "isn",
-                                errno, msg, written);
-    if (err)
-        PyErr_SetObject(PyExc_BlockingIOError, err);
-    Py_XDECREF(err);
-}
-
 /* Returns the address of the `written` member if a BlockingIOError was
    raised, NULL otherwise. The error is always re-raised. */
 static Py_ssize_t *
@@ -615,8 +580,7 @@ _buffered_raw_tell(buffered *self)
     if (n < 0) {
         if (!PyErr_Occurred())
             PyErr_Format(PyExc_IOError,
-                         "Raw stream returned invalid position %" PY_PRIdOFF,
-			 (PY_OFF_T_COMPAT)n);
+                         "Raw stream returned invalid position %zd", n);
         return -1;
     }
     self->abs_pos = n;
@@ -648,8 +612,7 @@ _buffered_raw_seek(buffered *self, Py_off_t target, int whence)
     if (n < 0) {
         if (!PyErr_Occurred())
             PyErr_Format(PyExc_IOError,
-                         "Raw stream returned invalid position %" PY_PRIdOFF,
-			 (PY_OFF_T_COMPAT)n);
+                         "Raw stream returned invalid position %zd", n);
         return -1;
     }
     self->abs_pos = n;
@@ -673,14 +636,11 @@ _buffered_init(buffered *self)
         return -1;
     }
 #ifdef WITH_THREAD
-    if (self->lock)
-        PyThread_free_lock(self->lock);
     self->lock = PyThread_allocate_lock();
     if (self->lock == NULL) {
         PyErr_SetString(PyExc_RuntimeError, "can't allocate read lock");
         return -1;
     }
-    self->owner = 0;
 #endif
     /* Find out whether buffer_size is a power of 2 */
     /* XXX is this optimization useful? */
@@ -695,65 +655,10 @@ _buffered_init(buffered *self)
     return 0;
 }
 
-/* Return 1 if an EnvironmentError with errno == EINTR is set (and then
-   clears the error indicator), 0 otherwise.
-   Should only be called when PyErr_Occurred() is true.
-*/
-static int
-_trap_eintr(void)
-{
-    static PyObject *eintr_int = NULL;
-    PyObject *typ, *val, *tb;
-    PyEnvironmentErrorObject *env_err;
-
-    if (eintr_int == NULL) {
-        eintr_int = PyLong_FromLong(EINTR);
-        assert(eintr_int != NULL);
-    }
-    if (!PyErr_ExceptionMatches(PyExc_EnvironmentError))
-        return 0;
-    PyErr_Fetch(&typ, &val, &tb);
-    PyErr_NormalizeException(&typ, &val, &tb);
-    env_err = (PyEnvironmentErrorObject *) val;
-    assert(env_err != NULL);
-    if (env_err->myerrno != NULL &&
-        PyObject_RichCompareBool(env_err->myerrno, eintr_int, Py_EQ) > 0) {
-        Py_DECREF(typ);
-        Py_DECREF(val);
-        Py_XDECREF(tb);
-        return 1;
-    }
-    /* This silences any error set by PyObject_RichCompareBool() */
-    PyErr_Restore(typ, val, tb);
-    return 0;
-}
-
 /*
  * Shared methods and wrappers
  */
 
-static PyObject *
-buffered_flush_and_rewind_unlocked(buffered *self)
-{
-    PyObject *res;
-
-    res = _bufferedwriter_flush_unlocked(self);
-    if (res == NULL)
-        return NULL;
-    Py_DECREF(res);
-
-    if (self->readable) {
-        /* Rewind the raw stream so that its position corresponds to
-           the current logical position. */
-        Py_off_t n;
-        n = _buffered_raw_seek(self, -RAW_OFFSET(self), 1);
-        _bufferedreader_reset_buf(self);
-        if (n == -1)
-            return NULL;
-    }
-    Py_RETURN_NONE;
-}
-
 static PyObject *
 buffered_flush(buffered *self, PyObject *args)
 {
@@ -762,9 +667,17 @@ buffered_flush(buffered *self, PyObject *args)
     CHECK_INITIALIZED(self)
     CHECK_CLOSED(self, "flush of closed file")
 
-    if (!ENTER_BUFFERED(self))
-        return NULL;
-    res = buffered_flush_and_rewind_unlocked(self);
+    ENTER_BUFFERED(self)
+    res = _bufferedwriter_flush_unlocked(self, 0);
+    if (res != NULL && self->readable) {
+        /* Rewind the raw stream so that its position corresponds to
+           the current logical position. */
+        Py_off_t n;
+        n = _buffered_raw_seek(self, -RAW_OFFSET(self), 1);
+        if (n == -1)
+            Py_CLEAR(res);
+        _bufferedreader_reset_buf(self);
+    }
     LEAVE_BUFFERED(self)
 
     return res;
@@ -781,11 +694,10 @@ buffered_peek(buffered *self, PyObject *args)
         return NULL;
     }
 
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
 
     if (self->writable) {
-        res = buffered_flush_and_rewind_unlocked(self);
+        res = _bufferedwriter_flush_unlocked(self, 1);
         if (res == NULL)
             goto end;
         Py_CLEAR(res);
@@ -804,7 +716,7 @@ buffered_read(buffered *self, PyObject *args)
     PyObject *res;
 
     CHECK_INITIALIZED(self)
-    if (!PyArg_ParseTuple(args, "|O&:read", &_PyIO_ConvertSsize_t, &n)) {
+    if (!PyArg_ParseTuple(args, "|n:read", &n)) {
         return NULL;
     }
     if (n < -1) {
@@ -817,21 +729,20 @@ buffered_read(buffered *self, PyObject *args)
 
     if (n == -1) {
         /* The number of bytes is unspecified, read until the end of stream */
-        if (!ENTER_BUFFERED(self))
-            return NULL;
+        ENTER_BUFFERED(self)
         res = _bufferedreader_read_all(self);
+        LEAVE_BUFFERED(self)
     }
     else {
         res = _bufferedreader_read_fast(self, n);
-        if (res != Py_None)
-            return res;
-        Py_DECREF(res);
-        if (!ENTER_BUFFERED(self))
-            return NULL;
-        res = _bufferedreader_read_generic(self, n);
+        if (res == Py_None) {
+            Py_DECREF(res);
+            ENTER_BUFFERED(self)
+            res = _bufferedreader_read_generic(self, n);
+            LEAVE_BUFFERED(self)
+        }
     }
 
-    LEAVE_BUFFERED(self)
     return res;
 }
 
@@ -854,9 +765,15 @@ buffered_read1(buffered *self, PyObject *args)
     if (n == 0)
         return PyBytes_FromStringAndSize(NULL, 0);
 
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
     
+    if (self->writable) {
+        res = _bufferedwriter_flush_unlocked(self, 1);
+        if (res == NULL)
+            goto end;
+        Py_CLEAR(res);
+    }
+
     /* Return up to n bytes.  If at least one byte is buffered, we
        only return buffered bytes.  Otherwise, we do one raw read. */
 
@@ -876,13 +793,6 @@ buffered_read1(buffered *self, PyObject *args)
         goto end;
     }
 
-    if (self->writable) {
-        res = buffered_flush_and_rewind_unlocked(self);
-        if (res == NULL)
-            goto end;
-        Py_DECREF(res);
-    }
-
     /* Fill the buffer from the raw stream, and copy it to the result. */
     _bufferedreader_reset_buf(self);
     r = _bufferedreader_fill_buffer(self);
@@ -905,10 +815,23 @@ end:
 static PyObject *
 buffered_readinto(buffered *self, PyObject *args)
 {
+    PyObject *res = NULL;
+
     CHECK_INITIALIZED(self)
     
-    /* TODO: use raw.readinto() (or a direct copy from our buffer) instead! */
-    return bufferediobase_readinto((PyObject *)self, args);
+    /* TODO: use raw.readinto() instead! */
+    if (self->writable) {
+        ENTER_BUFFERED(self)
+        res = _bufferedwriter_flush_unlocked(self, 0);
+        LEAVE_BUFFERED(self)
+        if (res == NULL)
+            goto end;
+        Py_DECREF(res);
+    }
+    res = bufferediobase_readinto((PyObject *)self, args);
+
+end:
+    return res;
 }
 
 static PyObject *
@@ -942,10 +865,15 @@ _buffered_readline(buffered *self, Py_ssize_t limit)
         goto end_unlocked;
     }
 
-    if (!ENTER_BUFFERED(self))
-        goto end_unlocked;
+    ENTER_BUFFERED(self)
 
     /* Now we try to get some more from the raw stream */
+    if (self->writable) {
+        res = _bufferedwriter_flush_unlocked(self, 1);
+        if (res == NULL)
+            goto end;
+        Py_CLEAR(res);
+    }
     chunks = PyList_New(0);
     if (chunks == NULL)
         goto end;
@@ -959,16 +887,9 @@ _buffered_readline(buffered *self, Py_ssize_t limit)
         }
         Py_CLEAR(res);
         written += n;
-        self->pos += n;
         if (limit >= 0)
             limit -= n;
     }
-    if (self->writable) {
-        PyObject *r = buffered_flush_and_rewind_unlocked(self);
-        if (r == NULL)
-            goto end;
-        Py_DECREF(r);
-    }
 
     for (;;) {
         _bufferedreader_reset_buf(self);
@@ -1028,8 +949,10 @@ buffered_readline(buffered *self, PyObject *args)
     Py_ssize_t limit = -1;
 
     CHECK_INITIALIZED(self)
-    if (!PyArg_ParseTuple(args, "|O&:readline", &_PyIO_ConvertSsize_t, &limit))
+
+    if (!PyArg_ParseTuple(args, "|n:readline", &limit)) {
         return NULL;
+    }
     return _buffered_readline(self, limit);
 }
 
@@ -1094,12 +1017,11 @@ buffered_seek(buffered *self, PyObject *args)
         }
     }
 
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
 
     /* Fallback: invoke raw seek() method and clear buffer */
     if (self->writable) {
-        res = _bufferedwriter_flush_unlocked(self);
+        res = _bufferedwriter_flush_unlocked(self, 0);
         if (res == NULL)
             goto end;
         Py_CLEAR(res);
@@ -1133,15 +1055,23 @@ buffered_truncate(buffered *self, PyObject *args)
         return NULL;
     }
 
-    if (!ENTER_BUFFERED(self))
-        return NULL;
+    ENTER_BUFFERED(self)
 
     if (self->writable) {
-        res = buffered_flush_and_rewind_unlocked(self);
+        res = _bufferedwriter_flush_unlocked(self, 0);
         if (res == NULL)
             goto end;
         Py_CLEAR(res);
     }
+    if (self->readable) {
+        if (pos == Py_None) {
+            /* Rewind the raw stream so that its position corresponds to
+               the current logical position. */
+            if (_buffered_raw_seek(self, -RAW_OFFSET(self), 1) == -1)
+                goto end;
+        }
+        _bufferedreader_reset_buf(self);
+    }
     res = PyObject_CallMethodObjArgs(self->raw, _PyIO_str_truncate, pos, NULL);
     if (res == NULL)
         goto end;
@@ -1203,17 +1133,12 @@ buffered_repr(buffered *self)
             PyErr_Clear();
         else
             return NULL;
-        res = PyString_FromFormat("<%s>", Py_TYPE(self)->tp_name);
+        res = PyUnicode_FromFormat("<%s>", Py_TYPE(self)->tp_name);
     }
     else {
-        PyObject *repr = PyObject_Repr(nameobj);
+        res = PyUnicode_FromFormat("<%s name=%R>",
+                                   Py_TYPE(self)->tp_name, nameobj);
         Py_DECREF(nameobj);
-        if (repr == NULL)
-            return NULL;
-        res = PyString_FromFormat("<%s name=%s>",
-                                   Py_TYPE(self)->tp_name,
-                                   PyString_AS_STRING(repr));
-        Py_DECREF(repr);
     }
     return res;
 }
@@ -1278,14 +1203,7 @@ _bufferedreader_raw_read(buffered *self, char *start, Py_ssize_t len)
     memobj = PyMemoryView_FromBuffer(&buf);
     if (memobj == NULL)
         return -1;
-    /* NOTE: PyErr_SetFromErrno() calls PyErr_CheckSignals() when EINTR
-       occurs so we needn't do it ourselves.
-       We then retry reading, ignoring the signal if no handler has
-       raised (see issue #10956).
-    */
-    do {
-        res = PyObject_CallMethodObjArgs(self->raw, _PyIO_str_readinto, memobj, NULL);
-    } while (res == NULL && _trap_eintr());
+    res = PyObject_CallMethodObjArgs(self->raw, _PyIO_str_readinto, memobj, NULL);
     Py_DECREF(memobj);
     if (res == NULL)
         return -1;
@@ -1343,18 +1261,17 @@ _bufferedreader_read_all(buffered *self)
             Py_DECREF(chunks);
             return NULL;
         }
-        self->pos += current_size;
     }
+    _bufferedreader_reset_buf(self);
     /* We're going past the buffer's bounds, flush it */
     if (self->writable) {
-        res = buffered_flush_and_rewind_unlocked(self);
+        res = _bufferedwriter_flush_unlocked(self, 1);
         if (res == NULL) {
             Py_DECREF(chunks);
             return NULL;
         }
         Py_CLEAR(res);
     }
-    _bufferedreader_reset_buf(self);
     while (1) {
         if (data) {
             if (PyList_Append(chunks, data) < 0) {
@@ -1437,14 +1354,6 @@ _bufferedreader_read_generic(buffered *self, Py_ssize_t n)
         memcpy(out, self->buffer + self->pos, current_size);
         remaining -= current_size;
         written += current_size;
-        self->pos += current_size;
-    }
-    /* Flush the write buffer if necessary */
-    if (self->writable) {
-        PyObject *r = buffered_flush_and_rewind_unlocked(self);
-        if (r == NULL)
-            goto error;
-        Py_DECREF(r);
     }
     _bufferedreader_reset_buf(self);
     while (remaining > 0) {
@@ -1474,10 +1383,7 @@ _bufferedreader_read_generic(buffered *self, Py_ssize_t n)
     self->pos = 0;
     self->raw_pos = 0;
     self->read_end = 0;
-    /* NOTE: when the read is satisfied, we avoid issuing any additional
-       reads, which could block indefinitely (e.g. on a socket).
-       See issue #9550. */
-    while (remaining > 0 && self->read_end < self->buffer_size) {
+    while (self->read_end < self->buffer_size) {
         Py_ssize_t r = _bufferedreader_fill_buffer(self);
         if (r == -1)
             goto error;
@@ -1564,7 +1470,7 @@ static PyMethodDef bufferedreader_methods[] = {
 };
 
 static PyMemberDef bufferedreader_members[] = {
-    {"raw", T_OBJECT, offsetof(buffered, raw), READONLY},
+    {"raw", T_OBJECT, offsetof(buffered, raw), 0},
     {NULL}
 };
 
@@ -1695,34 +1601,16 @@ _bufferedwriter_raw_write(buffered *self, char *start, Py_ssize_t len)
     Py_buffer buf;
     PyObject *memobj, *res;
     Py_ssize_t n;
-    int errnum;
     /* NOTE: the buffer needn't be released as its object is NULL. */
     if (PyBuffer_FillInfo(&buf, NULL, start, len, 1, PyBUF_CONTIG_RO) == -1)
         return -1;
     memobj = PyMemoryView_FromBuffer(&buf);
     if (memobj == NULL)
         return -1;
-    /* NOTE: PyErr_SetFromErrno() calls PyErr_CheckSignals() when EINTR
-       occurs so we needn't do it ourselves.
-       We then retry writing, ignoring the signal if no handler has
-       raised (see issue #10956).
-    */
-    do {
-        errno = 0;
-        res = PyObject_CallMethodObjArgs(self->raw, _PyIO_str_write, memobj, NULL);
-        errnum = errno;
-    } while (res == NULL && _trap_eintr());
+    res = PyObject_CallMethodObjArgs(self->raw, _PyIO_str_write, memobj, NULL);
     Py_DECREF(memobj);
     if (res == NULL)
         return -1;
-    if (res == Py_None) {
-        /* Non-blocking stream would have blocked. Special return code!
-           Being paranoid we reset errno in case it is changed by code
-           triggered by a decref.  errno is used by _set_BlockingIOError(). */
-        Py_DECREF(res);
-        errno = errnum;
-        return -2;
-    }
     n = PyNumber_AsSsize_t(res, PyExc_ValueError);
     Py_DECREF(res);
     if (n < 0 || n > len) {
@@ -1739,7 +1627,7 @@ _bufferedwriter_raw_write(buffered *self, char *start, Py_ssize_t len)
 /* `restore_pos` is 1 if we need to restore the raw stream position at
    the end, 0 otherwise. */
 static PyObject *
-_bufferedwriter_flush_unlocked(buffered *self)
+_bufferedwriter_flush_unlocked(buffered *self, int restore_pos)
 {
     Py_ssize_t written = 0;
     Py_off_t n, rewind;
@@ -1761,23 +1649,31 @@ _bufferedwriter_flush_unlocked(buffered *self)
             Py_SAFE_DOWNCAST(self->write_end - self->write_pos,
                              Py_off_t, Py_ssize_t));
         if (n == -1) {
-            goto error;
-        }
-        else if (n == -2) {
-            _set_BlockingIOError("write could not complete without blocking",
-                                 0);
+            Py_ssize_t *w = _buffered_check_blocking_error();
+            if (w == NULL)
+                goto error;
+            self->write_pos += *w;
+            self->raw_pos = self->write_pos;
+            written += *w;
+            *w = written;
+            /* Already re-raised */
             goto error;
         }
         self->write_pos += n;
         self->raw_pos = self->write_pos;
         written += Py_SAFE_DOWNCAST(n, Py_off_t, Py_ssize_t);
-        /* Partial writes can return successfully when interrupted by a
-           signal (see write(2)).  We must run signal handlers before
-           blocking another time, possibly indefinitely. */
-        if (PyErr_CheckSignals() < 0)
-            goto error;
     }
 
+    if (restore_pos) {
+        Py_off_t forward = rewind - written;
+        if (forward != 0) {
+            n = _buffered_raw_seek(self, forward, 1);
+            if (n < 0) {
+                goto error;
+            }
+            self->raw_pos += forward;
+        }
+    }
     _bufferedwriter_reset_buf(self);
 
 end:
@@ -1792,11 +1688,10 @@ bufferedwriter_write(buffered *self, PyObject *args)
 {
     PyObject *res = NULL;
     Py_buffer buf;
-    Py_ssize_t written, avail, remaining;
-    Py_off_t offset;
+    Py_ssize_t written, avail, remaining, n;
 
     CHECK_INITIALIZED(self)
-    if (!PyArg_ParseTuple(args, "s*:write", &buf)) {
+    if (!PyArg_ParseTuple(args, "y*:write", &buf)) {
         return NULL;
     }
 
@@ -1806,10 +1701,7 @@ bufferedwriter_write(buffered *self, PyObject *args)
         return NULL;
     }
 
-    if (!ENTER_BUFFERED(self)) {
-        PyBuffer_Release(&buf);
-        return NULL;
-    }
+    ENTER_BUFFERED(self)
 
     /* Fast path: the data to write can be fully buffered. */
     if (!VALID_READ_BUFFER(self) && !VALID_WRITE_BUFFER(self)) {
@@ -1819,7 +1711,7 @@ bufferedwriter_write(buffered *self, PyObject *args)
     avail = Py_SAFE_DOWNCAST(self->buffer_size - self->pos, Py_off_t, Py_ssize_t);
     if (buf.len <= avail) {
         memcpy(self->buffer + self->pos, buf.buf, buf.len);
-        if (!VALID_WRITE_BUFFER(self) || self->write_pos > self->pos) {
+        if (!VALID_WRITE_BUFFER(self)) {
             self->write_pos = self->pos;
         }
         ADJUST_POSITION(self, self->pos + buf.len);
@@ -1830,7 +1722,7 @@ bufferedwriter_write(buffered *self, PyObject *args)
     }
 
     /* First write the current buffer */
-    res = _bufferedwriter_flush_unlocked(self);
+    res = _bufferedwriter_flush_unlocked(self, 0);
     if (res == NULL) {
         Py_ssize_t *w = _buffered_check_blocking_error();
         if (w == NULL)
@@ -1853,19 +1745,14 @@ bufferedwriter_write(buffered *self, PyObject *args)
             PyErr_Clear();
             memcpy(self->buffer + self->write_end, buf.buf, buf.len);
             self->write_end += buf.len;
-            self->pos += buf.len;
             written = buf.len;
             goto end;
         }
         /* Buffer as much as possible. */
         memcpy(self->buffer + self->write_end, buf.buf, avail);
         self->write_end += avail;
-        self->pos += avail;
-        /* XXX Modifying the existing exception e using the pointer w
-           will change e.characters_written but not e.args[2].
-           Therefore we just replace with a new error. */
-        _set_BlockingIOError("write could not complete without blocking",
-                             avail);
+        /* Already re-raised */
+        *w = avail;
         goto error;
     }
     Py_CLEAR(res);
@@ -1876,23 +1763,25 @@ bufferedwriter_write(buffered *self, PyObject *args)
        the raw stream by itself).
        Fixes issue #6629.
     */
-    offset = RAW_OFFSET(self);
-    if (offset != 0) {
-        if (_buffered_raw_seek(self, -offset, 1) < 0)
+    n = RAW_OFFSET(self);
+    if (n != 0) {
+        if (_buffered_raw_seek(self, -n, 1) < 0)
             goto error;
-        self->raw_pos -= offset;
+        self->raw_pos -= n;
     }
 
     /* Then write buf itself. At this point the buffer has been emptied. */
     remaining = buf.len;
     written = 0;
     while (remaining > self->buffer_size) {
-        Py_ssize_t n = _bufferedwriter_raw_write(
+        n = _bufferedwriter_raw_write(
             self, (char *) buf.buf + written, buf.len - written);
         if (n == -1) {
-            goto error;
-        } else if (n == -2) {
-            /* Write failed because raw file is non-blocking */
+            Py_ssize_t *w = _buffered_check_blocking_error();
+            if (w == NULL)
+                goto error;
+            written += *w;
+            remaining -= *w;
             if (remaining > self->buffer_size) {
                 /* Can't buffer everything, still buffer as much as possible */
                 memcpy(self->buffer,
@@ -1900,9 +1789,8 @@ bufferedwriter_write(buffered *self, PyObject *args)
                 self->raw_pos = 0;
                 ADJUST_POSITION(self, self->buffer_size);
                 self->write_end = self->buffer_size;
-                written += self->buffer_size;
-                _set_BlockingIOError("write could not complete without "
-                                     "blocking", written);
+                *w = written + self->buffer_size;
+                /* Already re-raised */
                 goto error;
             }
             PyErr_Clear();
@@ -1910,11 +1798,6 @@ bufferedwriter_write(buffered *self, PyObject *args)
         }
         written += n;
         remaining -= n;
-        /* Partial writes can return successfully when interrupted by a
-           signal (see write(2)).  We must run signal handlers before
-           blocking another time, possibly indefinitely. */
-        if (PyErr_CheckSignals() < 0)
-            goto error;
     }
     if (self->readable)
         _bufferedreader_reset_buf(self);
@@ -1956,7 +1839,7 @@ static PyMethodDef bufferedwriter_methods[] = {
 };
 
 static PyMemberDef bufferedwriter_members[] = {
-    {"raw", T_OBJECT, offsetof(buffered, raw), READONLY},
+    {"raw", T_OBJECT, offsetof(buffered, raw), 0},
     {NULL}
 };
 
@@ -2193,11 +2076,6 @@ bufferedrwpair_isatty(rwpair *self, PyObject *args)
 static PyObject *
 bufferedrwpair_closed_get(rwpair *self, void *context)
 {
-    if (self->writer == NULL) {
-        PyErr_SetString(PyExc_RuntimeError,
-                "the BufferedRWPair object is being garbage-collected");
-        return NULL;
-    }
     return PyObject_GetAttr((PyObject *) self->writer, _PyIO_str_closed);
 }
 
@@ -2351,7 +2229,7 @@ static PyMethodDef bufferedrandom_methods[] = {
 };
 
 static PyMemberDef bufferedrandom_members[] = {
-    {"raw", T_OBJECT, offsetof(buffered, raw), READONLY},
+    {"raw", T_OBJECT, offsetof(buffered, raw), 0},
     {NULL}
 };
 
