@@ -41,15 +41,15 @@ PyDoc_STRVAR(iobase_doc,
     "bytes. bytearrays are accepted too, and in some cases (such as\n"
     "readinto) needed. Text I/O classes work with str data.\n"
     "\n"
-    "Note that calling any method (except additional calls to close(),\n"
-    "which are ignored) on a closed stream should raise a ValueError.\n"
+    "Note that calling any method (even inquiries) on a closed stream is\n"
+    "undefined. Implementations may raise IOError in this case.\n"
     "\n"
     "IOBase (and its subclasses) support the iterator protocol, meaning\n"
     "that an IOBase object can be iterated over yielding the lines in a\n"
     "stream.\n"
     "\n"
     "IOBase also supports the :keyword:`with` statement. In this example,\n"
-    "fp is closed after the suite of the with statement is complete:\n"
+    "fp is closed after the suite of the with statment is complete:\n"
     "\n"
     "with open('spam.txt', 'r') as fp:\n"
     "    fp.write('Spam and eggs!')\n");
@@ -65,7 +65,7 @@ PyDoc_STRVAR(iobase_doc,
 static PyObject *
 iobase_unsupported(const char *message)
 {
-    PyErr_SetString(_PyIO_unsupported_operation, message);
+    PyErr_SetString(IO_STATE->unsupported_operation, message);
     return NULL;
 }
 
@@ -74,7 +74,7 @@ iobase_unsupported(const char *message)
 PyDoc_STRVAR(iobase_seek_doc,
     "Change stream position.\n"
     "\n"
-    "Change the stream position to the given byte offset. The offset is\n"
+    "Change the stream position to byte offset offset. offset is\n"
     "interpreted relative to the position indicated by whence.  Values\n"
     "for whence are:\n"
     "\n"
@@ -102,8 +102,8 @@ iobase_tell(PyObject *self, PyObject *args)
 PyDoc_STRVAR(iobase_truncate_doc,
     "Truncate file to size bytes.\n"
     "\n"
-    "File pointer is left unchanged.  Size defaults to the current IO\n"
-    "position as reported by tell().  Returns the new size.");
+    "Size defaults to the current IO position as reported by tell().  Return\n"
+    "the new size.");
 
 static PyObject *
 iobase_truncate(PyObject *self, PyObject *args)
@@ -183,7 +183,11 @@ iobase_close(PyObject *self, PyObject *args)
     res = PyObject_CallMethodObjArgs(self, _PyIO_str_flush, NULL);
     PyObject_SetAttrString(self, "__IOBase_closed", Py_True);
     if (res == NULL) {
-        return NULL;
+        /* If flush() fails, just give up */
+        if (PyErr_ExceptionMatches(PyExc_IOError))
+            PyErr_Clear();
+        else
+            return NULL;
     }
     Py_XDECREF(res);
     Py_RETURN_NONE;
@@ -437,7 +441,7 @@ PyDoc_STRVAR(iobase_readline_doc,
     "\n"
     "If limit is specified, at most limit bytes will be read.\n"
     "\n"
-    "The line terminator is always b'\\n' for binary files; for text\n"
+    "The line terminator is always b'\n' for binary files; for text\n"
     "files, the newlines argument to open can be used to select the line\n"
     "terminator(s) recognized.\n");
 
@@ -451,7 +455,7 @@ iobase_readline(PyObject *self, PyObject *args)
     PyObject *buffer, *result;
     Py_ssize_t old_size = -1;
 
-    if (!PyArg_ParseTuple(args, "|O&:readline", &_PyIO_ConvertSsize_t, &limit)) {
+    if (!PyArg_ParseTuple(args, "|n:readline", &limit)) {
         return NULL;
     }
 
@@ -468,14 +472,8 @@ iobase_readline(PyObject *self, PyObject *args)
 
         if (has_peek) {
             PyObject *readahead = PyObject_CallMethod(self, "peek", "i", 1);
-            if (readahead == NULL) {
-                /* NOTE: PyErr_SetFromErrno() calls PyErr_CheckSignals()
-                   when EINTR occurs so we needn't do it ourselves. */
-                if (_PyIO_trap_eintr()) {
-                    continue;
-                }
+            if (readahead == NULL)
                 goto fail;
-            }
             if (!PyBytes_Check(readahead)) {
                 PyErr_Format(PyExc_IOError,
                              "peek() should have returned a bytes object, "
@@ -508,14 +506,8 @@ iobase_readline(PyObject *self, PyObject *args)
         }
 
         b = PyObject_CallMethod(self, "read", "n", nreadahead);
-        if (b == NULL) {
-            /* NOTE: PyErr_SetFromErrno() calls PyErr_CheckSignals()
-               when EINTR occurs so we needn't do it ourselves. */
-            if (_PyIO_trap_eintr()) {
-                continue;
-            }
+        if (b == NULL)
             goto fail;
-        }
         if (!PyBytes_Check(b)) {
             PyErr_Format(PyExc_IOError,
                          "read() should have returned a bytes object, "
@@ -585,11 +577,16 @@ static PyObject *
 iobase_readlines(PyObject *self, PyObject *args)
 {
     Py_ssize_t hint = -1, length = 0;
-    PyObject *result;
+    PyObject *hintobj = Py_None, *result;
 
-    if (!PyArg_ParseTuple(args, "|O&:readlines", &_PyIO_ConvertSsize_t, &hint)) {
+    if (!PyArg_ParseTuple(args, "|O:readlines", &hintobj)) {
         return NULL;
     }
+    if (hintobj != Py_None) {
+        hint = PyNumber_AsSsize_t(hintobj, PyExc_ValueError);
+        if (hint == -1 && PyErr_Occurred())
+            return NULL;
+    }
 
     result = PyList_New(0);
     if (result == NULL)
@@ -660,10 +657,7 @@ iobase_writelines(PyObject *self, PyObject *args)
                 break; /* Stop Iteration */
         }
 
-        res = NULL;
-        do {
-            res = PyObject_CallMethodObjArgs(self, _PyIO_str_write, line, NULL);
-        } while (res == NULL && _PyIO_trap_eintr());
+        res = PyObject_CallMethodObjArgs(self, _PyIO_str_write, line, NULL);
         Py_DECREF(line);
         if (res == NULL) {
             Py_DECREF(iter);
@@ -791,9 +785,9 @@ rawiobase_read(PyObject *self, PyObject *args)
         return NULL;
 
     res = PyObject_CallMethodObjArgs(self, _PyIO_str_readinto, b, NULL);
-    if (res == NULL || res == Py_None) {
+    if (res == NULL) {
         Py_DECREF(b);
-        return res;
+        return NULL;
     }
 
     n = PyNumber_AsSsize_t(res, PyExc_ValueError);
@@ -826,22 +820,9 @@ rawiobase_readall(PyObject *self, PyObject *args)
         PyObject *data = PyObject_CallMethod(self, "read",
                                              "i", DEFAULT_BUFFER_SIZE);
         if (!data) {
-            /* NOTE: PyErr_SetFromErrno() calls PyErr_CheckSignals()
-               when EINTR occurs so we needn't do it ourselves. */
-            if (_PyIO_trap_eintr()) {
-                continue;
-            }
             Py_DECREF(chunks);
             return NULL;
         }
-        if (data == Py_None) {
-            if (PyList_GET_SIZE(chunks) == 0) {
-                Py_DECREF(chunks);
-                return data;
-            }
-            Py_DECREF(data);
-            break;
-        }
         if (!PyBytes_Check(data)) {
             Py_DECREF(chunks);
             Py_DECREF(data);
