@@ -32,7 +32,7 @@ unknown_presentation_type(STRINGLIB_CHAR presentation_type,
         PyErr_Format(PyExc_ValueError,
                      "Unknown format code '%c' "
                      "for object of type '%.200s'",
-                     (char)presentation_type,
+                     presentation_type,
                      type_name);
 #if STRINGLIB_IS_UNICODE
     else
@@ -44,24 +44,6 @@ unknown_presentation_type(STRINGLIB_CHAR presentation_type,
 #endif
 }
 
-static void
-invalid_comma_type(STRINGLIB_CHAR presentation_type)
-{
-#if STRINGLIB_IS_UNICODE
-    /* See comment in unknown_presentation_type */
-    if (presentation_type > 32 && presentation_type < 128)
-#endif
-        PyErr_Format(PyExc_ValueError,
-                     "Cannot specify ',' with '%c'.",
-                     (char)presentation_type);
-#if STRINGLIB_IS_UNICODE
-    else
-        PyErr_Format(PyExc_ValueError,
-                     "Cannot specify ',' with '\\x%x'.",
-                     (unsigned int)presentation_type);
-#endif
-}
-
 /*
     get_integer consumes 0 or more decimal digit characters from an
     input string, updates *result with the corresponding positive
@@ -73,7 +55,7 @@ static int
 get_integer(STRINGLIB_CHAR **ptr, STRINGLIB_CHAR *end,
                   Py_ssize_t *result)
 {
-    Py_ssize_t accumulator, digitval;
+    Py_ssize_t accumulator, digitval, oldaccumulator;
     int numdigits;
     accumulator = numdigits = 0;
     for (;;(*ptr)++, numdigits++) {
@@ -83,17 +65,19 @@ get_integer(STRINGLIB_CHAR **ptr, STRINGLIB_CHAR *end,
         if (digitval < 0)
             break;
         /*
-           Detect possible overflow before it happens:
-
-              accumulator * 10 + digitval > PY_SSIZE_T_MAX if and only if
-              accumulator > (PY_SSIZE_T_MAX - digitval) / 10.
+           This trick was copied from old Unicode format code.  It's cute,
+           but would really suck on an old machine with a slow divide
+           implementation.  Fortunately, in the normal case we do not
+           expect too many digits.
         */
-        if (accumulator > (PY_SSIZE_T_MAX - digitval) / 10) {
+        oldaccumulator = accumulator;
+        accumulator *= 10;
+        if ((accumulator+10)/10 != oldaccumulator+1) {
             PyErr_Format(PyExc_ValueError,
                          "Too many decimal digits in format string");
             return -1;
         }
-        accumulator = accumulator * 10 + digitval;
+        accumulator += digitval;
     }
     *result = accumulator;
     return numdigits;
@@ -139,26 +123,6 @@ typedef struct {
     STRINGLIB_CHAR type;
 } InternalFormatSpec;
 
-
-#if 0
-/* Occassionally useful for debugging. Should normally be commented out. */
-static void
-DEBUG_PRINT_FORMAT_SPEC(InternalFormatSpec *format)
-{
-    printf("internal format spec: fill_char %d\n", format->fill_char);
-    printf("internal format spec: align %d\n", format->align);
-    printf("internal format spec: alternate %d\n", format->alternate);
-    printf("internal format spec: sign %d\n", format->sign);
-    printf("internal format spec: width %zd\n", format->width);
-    printf("internal format spec: thousands_separators %d\n",
-           format->thousands_separators);
-    printf("internal format spec: precision %zd\n", format->precision);
-    printf("internal format spec: type %c\n", format->type);
-    printf("\n");
-}
-#endif
-
-
 /*
   ptr points to the start of the format_spec, end points just past its end.
   fills in format with the parsed information.
@@ -169,8 +133,7 @@ static int
 parse_internal_render_format_spec(STRINGLIB_CHAR *format_spec,
                                   Py_ssize_t format_spec_len,
                                   InternalFormatSpec *format,
-                                  char default_type,
-                                  char default_align)
+                                  char default_type)
 {
     STRINGLIB_CHAR *ptr = format_spec;
     STRINGLIB_CHAR *end = format_spec + format_spec_len;
@@ -179,11 +142,9 @@ parse_internal_render_format_spec(STRINGLIB_CHAR *format_spec,
        the input string */
 
     Py_ssize_t consumed;
-    int align_specified = 0;
-    int fill_char_specified = 0;
 
-    format->fill_char = ' ';
-    format->align = default_align;
+    format->fill_char = '\0';
+    format->align = '\0';
     format->alternate = 0;
     format->sign = '\0';
     format->width = -1;
@@ -196,13 +157,10 @@ parse_internal_render_format_spec(STRINGLIB_CHAR *format_spec,
     if (end-ptr >= 2 && is_alignment_token(ptr[1])) {
         format->align = ptr[1];
         format->fill_char = ptr[0];
-        fill_char_specified = 1;
-        align_specified = 1;
         ptr += 2;
     }
     else if (end-ptr >= 1 && is_alignment_token(ptr[0])) {
         format->align = ptr[0];
-        align_specified = 1;
         ++ptr;
     }
 
@@ -220,9 +178,9 @@ parse_internal_render_format_spec(STRINGLIB_CHAR *format_spec,
     }
 
     /* The special case for 0-padding (backwards compat) */
-    if (!fill_char_specified && end-ptr >= 1 && ptr[0] == '0') {
+    if (format->fill_char == '\0' && end-ptr >= 1 && ptr[0] == '0') {
         format->fill_char = '0';
-        if (!align_specified) {
+        if (format->align == '\0') {
             format->align = '=';
         }
         ++ptr;
@@ -295,7 +253,8 @@ parse_internal_render_format_spec(STRINGLIB_CHAR *format_spec,
             /* These are allowed. See PEP 378.*/
             break;
         default:
-            invalid_comma_type(format->type);
+            PyErr_Format(PyExc_ValueError,
+                         "Cannot specify ',' with '%c'.", format->type);
             return 0;
         }
     }
@@ -320,19 +279,14 @@ calc_padding(Py_ssize_t nchars, Py_ssize_t width, STRINGLIB_CHAR align,
         *n_total = nchars;
     }
 
-    /* Figure out how much leading space we need, based on the
+    /* figure out how much leading space we need, based on the
        aligning */
     if (align == '>')
         *n_lpadding = *n_total - nchars;
     else if (align == '^')
         *n_lpadding = (*n_total - nchars) / 2;
-    else if (align == '<' || align == '=')
+    else
         *n_lpadding = 0;
-    else {
-        /* We should never have an unspecified alignment. */
-        *n_lpadding = 0;
-        assert(0);
-    }
 
     *n_rpadding = *n_total - nchars - *n_lpadding;
 }
@@ -498,7 +452,7 @@ calc_number_widths(NumberFieldWidths *spec, Py_ssize_t n_prefix,
 
     /* min_width can go negative, that's okay. format->width == -1 means
        we don't care. */
-    if (format->fill_char == '0' && format->align == '=')
+    if (format->fill_char == '0')
         spec->n_min_width = format->width - n_non_digit_non_padding;
     else
         spec->n_min_width = 0;
@@ -534,13 +488,9 @@ calc_number_widths(NumberFieldWidths *spec, Py_ssize_t n_prefix,
         case '=':
             spec->n_spadding = n_padding;
             break;
-        case '>':
-            spec->n_lpadding = n_padding;
-            break;
         default:
-            /* Shouldn't get here, but treat it as '>' */
+            /* Handles '>', plus catch-all just in case. */
             spec->n_lpadding = n_padding;
-            assert(0);
             break;
         }
     }
@@ -649,8 +599,8 @@ get_locale_info(int type, LocaleInfo *locale_info)
     case LT_DEFAULT_LOCALE:
         locale_info->decimal_point = ".";
         locale_info->thousands_sep = ",";
-        locale_info->grouping = "\3"; /* Group every 3 characters.  The
-                                         (implicit) trailing 0 means repeat
+        locale_info->grouping = "\3"; /* Group every 3 characters,
+                                         trailing 0 means repeat
                                          infinitely. */
         break;
     case LT_NO_LOCALE:
@@ -717,7 +667,8 @@ format_string_internal(PyObject *value, const InternalFormatSpec *format)
 
     /* Write into that space. First the padding. */
     p = fill_padding(STRINGLIB_STR(result), len,
-                     format->fill_char, lpad, rpad);
+                     format->fill_char=='\0'?' ':format->fill_char,
+                     lpad, rpad);
 
     /* Then the source string. */
     memcpy(p, STRINGLIB_STR(value), len * sizeof(STRINGLIB_CHAR));
@@ -789,7 +740,6 @@ format_int_or_long_internal(PyObject *value, const InternalFormatSpec *format,
         x = PyLong_AsLong(value);
         if (x == -1 && PyErr_Occurred())
             goto done;
-#if STRINGLIB_IS_UNICODE
 #ifdef Py_UNICODE_WIDE
         if (x < 0 || x > 0x10ffff) {
             PyErr_SetString(PyExc_OverflowError,
@@ -804,13 +754,6 @@ format_int_or_long_internal(PyObject *value, const InternalFormatSpec *format,
                             "(narrow Python build)");
             goto done;
         }
-#endif
-#else
-        if (x < 0 || x > 0xff) {
-            PyErr_SetString(PyExc_OverflowError,
-                            "%c arg not in range(0x100)");
-            goto done;
-        }
 #endif
         numeric_char = (STRINGLIB_CHAR)x;
         pnumeric_chars = &numeric_char;
@@ -902,7 +845,8 @@ format_int_or_long_internal(PyObject *value, const InternalFormatSpec *format,
 
     /* Populate the memory. */
     fill_number(STRINGLIB_STR(result), &spec, pnumeric_chars, n_digits,
-                prefix, format->fill_char, &locale, format->type == 'X');
+                prefix, format->fill_char == '\0' ? ' ' : format->fill_char,
+                &locale, format->type == 'X');
 
 done:
     Py_XDECREF(tmp);
@@ -936,7 +880,7 @@ format_float_internal(PyObject *value,
     Py_ssize_t n_total;
     int has_decimal;
     double val;
-    Py_ssize_t precision;
+    Py_ssize_t precision = format->precision;
     Py_ssize_t default_precision = 6;
     STRINGLIB_CHAR type = format->type;
     int add_pct = 0;
@@ -955,12 +899,6 @@ format_float_internal(PyObject *value,
        from a hard-code pseudo-locale */
     LocaleInfo locale;
 
-    if (format->precision > INT_MAX) {
-        PyErr_SetString(PyExc_ValueError, "precision too big");
-        goto done;
-    }
-    precision = (int)format->precision;
-
     /* Alternate is not allowed on floats. */
     if (format->alternate) {
         PyErr_SetString(PyExc_ValueError,
@@ -982,6 +920,13 @@ format_float_internal(PyObject *value,
            format the result. We take care of that later. */
         type = 'g';
 
+#if PY_VERSION_HEX < 0x0301000
+    /* 'F' is the same as 'f', per the PEP */
+    /* This is no longer the case in 3.x */
+    if (type == 'F')
+        type = 'f';
+#endif
+
     val = PyFloat_AsDouble(value);
     if (val == -1.0 && PyErr_Occurred())
         goto done;
@@ -995,6 +940,12 @@ format_float_internal(PyObject *value,
     if (precision < 0)
         precision = default_precision;
 
+#if PY_VERSION_HEX < 0x03010000
+    /* 3.1 no longer converts large 'f' to 'g'. */
+    if ((type == 'f' || type == 'F') && fabs(val) >= 1e50)
+        type = 'g';
+#endif
+
     /* Cast "type", because if we're in unicode we need to pass a
        8-bit char. This is safe, because we've restricted what "type"
        can be. */
@@ -1056,7 +1007,8 @@ format_float_internal(PyObject *value,
 
     /* Populate the memory. */
     fill_number(STRINGLIB_STR(result), &spec, p, n_digits, NULL,
-                format->fill_char, &locale, 0);
+                format->fill_char == '\0' ? ' ' : format->fill_char, &locale,
+                0);
 
 done:
     PyMem_Free(buf);
@@ -1091,7 +1043,7 @@ format_complex_internal(PyObject *value,
     Py_ssize_t n_im_total;
     int re_has_decimal;
     int im_has_decimal;
-    Py_ssize_t precision;
+    Py_ssize_t precision = format->precision;
     Py_ssize_t default_precision = 6;
     STRINGLIB_CHAR type = format->type;
     STRINGLIB_CHAR *p_re;
@@ -1120,12 +1072,6 @@ format_complex_internal(PyObject *value,
        from a hard-code pseudo-locale */
     LocaleInfo locale;
 
-    if (format->precision > INT_MAX) {
-        PyErr_SetString(PyExc_ValueError, "precision too big");
-        goto done;
-    }
-    precision = (int)format->precision;
-
     /* Alternate is not allowed on complex. */
     if (format->alternate) {
         PyErr_SetString(PyExc_ValueError,
@@ -1161,10 +1107,9 @@ format_complex_internal(PyObject *value,
         /* Omitted type specifier. Should be like str(self). */
         type = 'g';
         default_precision = PyFloat_STR_PRECISION;
-        if (re == 0.0 && copysign(1.0, re) == 1.0)
+        add_parens = 1;
+        if (re == 0.0)
             skip_re = 1;
-        else
-            add_parens = 1;
     }
 
     if (type == 'n')
@@ -1172,6 +1117,13 @@ format_complex_internal(PyObject *value,
            format the result. We take care of that later. */
         type = 'g';
 
+#if PY_VERSION_HEX < 0x03010000
+    /* This is no longer the case in 3.x */
+    /* 'F' is the same as 'f', per the PEP */
+    if (type == 'F')
+        type = 'f';
+#endif
+
     if (precision < 0)
         precision = default_precision;
 
@@ -1241,7 +1193,7 @@ format_complex_internal(PyObject *value,
     /* Turn off any padding. We'll do it later after we've composed
        the numbers without padding. */
     tmp_format.fill_char = '\0';
-    tmp_format.align = '<';
+    tmp_format.align = '\0';
     tmp_format.width = -1;
 
     /* Calculate how much memory we'll need. */
@@ -1249,11 +1201,8 @@ format_complex_internal(PyObject *value,
                                     n_re_digits, n_re_remainder,
                                     re_has_decimal, &locale, &tmp_format);
 
-    /* Same formatting, but always include a sign, unless the real part is
-     * going to be omitted, in which case we use whatever sign convention was
-     * requested by the original format. */
-    if (!skip_re)
-        tmp_format.sign = '+';
+    /* Same formatting, but always include a sign. */
+    tmp_format.sign = '+';
     n_im_total = calc_number_widths(&im_spec, 0, im_sign_char, p_im,
                                     n_im_digits, n_im_remainder,
                                     im_has_decimal, &locale, &tmp_format);
@@ -1272,7 +1221,8 @@ format_complex_internal(PyObject *value,
     /* Populate the memory. First, the padding. */
     p = fill_padding(STRINGLIB_STR(result),
                      n_re_total + n_im_total + 1 + add_parens * 2,
-                     format->fill_char, lpad, rpad);
+                     format->fill_char=='\0' ? ' ' : format->fill_char,
+                     lpad, rpad);
 
     if (add_parens)
         *p++ = '(';
@@ -1319,7 +1269,7 @@ FORMAT_STRING(PyObject *obj,
 
     /* parse the format_spec */
     if (!parse_internal_render_format_spec(format_spec, format_spec_len,
-                                           &format, 's', '<'))
+                                           &format, 's'))
         goto done;
 
     /* type conversion? */
@@ -1359,7 +1309,7 @@ format_int_or_long(PyObject* obj,
     /* parse the format_spec */
     if (!parse_internal_render_format_spec(format_spec,
                                            format_spec_len,
-                                           &format, 'd', '>'))
+                                           &format, 'd'))
         goto done;
 
     /* type conversion? */
@@ -1470,7 +1420,7 @@ FORMAT_FLOAT(PyObject *obj,
     /* parse the format_spec */
     if (!parse_internal_render_format_spec(format_spec,
                                            format_spec_len,
-                                           &format, '\0', '>'))
+                                           &format, '\0'))
         goto done;
 
     /* type conversion? */
@@ -1518,7 +1468,7 @@ FORMAT_COMPLEX(PyObject *obj,
     /* parse the format_spec */
     if (!parse_internal_render_format_spec(format_spec,
                                            format_spec_len,
-                                           &format, '\0', '>'))
+                                           &format, '\0'))
         goto done;
 
     /* type conversion? */
